"""
EventPool - 事件池核心实现

负责：
1. 任务队列管理和消息分发
2. 消费者管理和生命周期控制
3. 优先级队列处理
4. 离线Worker恢复机制

核心组件集成：
- MessageSender/Reader: 消息发送和读取（通过container）
- QueueRegistry: 队列注册管理
- Worker ID: 直接使用 app.worker_id（在 App._start 中生成）
"""

from ..utils.serializer import dumps_str, loads_str
import time
import threading
import logging
import asyncio
import json
from collections import defaultdict, deque, Counter
from typing import List, Optional, TYPE_CHECKING, Union
import traceback
import redis
from redis import asyncio as aioredis

from jettask.db.connector import get_sync_redis_client, get_async_redis_client

from ..utils.helpers import get_hostname
import os
from jettask.config.lua_scripts import LUA_SCRIPT_BATCH_SEND_EVENT

if TYPE_CHECKING:
    from ..core.task import Task
    from ..core.app import Jettask

logger = logging.getLogger('app')

UPDATE_MAX_OFFSET_LUA = """
local hash_key = KEYS[1]
local field = KEYS[2] 
local new_value = tonumber(ARGV[1])

local current = redis.call('HGET', hash_key, field)
if current == false or tonumber(current) < new_value then
    redis.call('HSET', hash_key, field, new_value)
    return 1
else
    return 0
end
"""

class EventPool(object):
    STATE_MACHINE_NAME = "STATE_MACHINE"
    TIMEOUT = 60 * 5

    def __init__(
        self,
        redis_client: redis.StrictRedis,
        async_redis_client: Optional[aioredis.StrictRedis] = None,
        task_event_queues=None,
        tasks: dict = None,
        queue_registry=None,  
        offline_recovery=None,  
        queues: list = None,
        redis_url: str = None,
        consumer_config: dict = None,
        redis_prefix: str = None,
        app: Optional["Jettask"] = None,
        worker_id: str = None,
    ) -> None:
        self.redis_client = redis_client

        self._async_redis_client = async_redis_client  
        self._binary_redis_client = None
        self._async_binary_redis_client = None

        self._redis_url = redis_url or 'redis://localhost:6379/0'
        self.redis_prefix = redis_prefix or 'jettask'
        self.app = app  
        self.worker_id = worker_id  
        self.task_event_queues = task_event_queues  

        self.tasks = tasks
        from jettask.utils.queue_matcher import separate_wildcard_and_static_queues

        self.wildcard_patterns, static_queues = separate_wildcard_and_static_queues(queues or [])
        self.queues = static_queues  
        self.wildcard_mode = len(self.wildcard_patterns) > 0  
        
        self.consumer_config = consumer_config or {}
        self.consumer_config['queues'] = queues or []
        self.consumer_config['redis_prefix'] = redis_prefix or 'jettask'
        self.consumer_config['redis_url'] = redis_url or 'redis://localhost:6379/0'

        self.queue_registry = queue_registry
        self.offline_recovery = offline_recovery

        self.prefixed_queues = {}


        self._broadcast_message_tracker = {}

        self.solo_routing_tasks = {}
        self.solo_running_state = {}
        self.solo_urgent_retry = {}
        self.batch_routing_tasks = {}
        self.task_scheduler = {}
        self.running_task_state_mappings = {}
        self.delay_tasks = []
        self.solo_agg_task = {}
        self.rlock = threading.RLock()
        self._claimed_message_ids = set()  
        self._stop_reading = False  
        self._queue_stop_flags = {queue: False for queue in (queues or [])}  


    async def record_group_info_async(self, queue: str, task_name: str, group_name: str, consumer_name: str):
        if not self.worker_id:
            logger.warning("Cannot record group info: worker_id not initialized")
            return
        try:
            if self.app and hasattr(self.app, 'worker_state'):
                await self.app.worker_state.record_group_info(
                    worker_id=self.worker_id,
                    queue=queue,
                    task_name=task_name,
                    group_name=group_name,
                    consumer_name=consumer_name,
                    redis_prefix=self.redis_prefix
                )
            else:
                logger.warning("Cannot record group info: worker_state not available")

        except Exception as e:
            logger.error(f"Error recording task group info: {e}", exc_info=True)

    def _put_task(self, event_queue: Union[deque, asyncio.Queue], task, urgent: bool = False):
        if isinstance(event_queue, deque):
            if urgent:
                event_queue.appendleft(task)
            else:
                event_queue.append(task)
        elif isinstance(event_queue, asyncio.Queue):
            pass
    
    async def _async_put_task(self, event_queue: asyncio.Queue, task, urgent: bool = False):
        await event_queue.put(task)

    def init_routing(self):
        for queue in self.queues:
            self.solo_agg_task[queue] = defaultdict(list)
            self.solo_routing_tasks[queue] = defaultdict(list)
            self.solo_running_state[queue]  = defaultdict(bool)
            self.batch_routing_tasks[queue]  = defaultdict(list)
            self.task_scheduler[queue] = defaultdict(int)
            self.running_task_state_mappings[queue] = defaultdict(dict)
            
    def get_prefixed_queue_name(self, queue: str) -> str:
        return f"{self.redis_prefix}:QUEUE:{queue}"

    @property
    def async_redis_client(self):
        if self._async_redis_client is None:
            self._async_redis_client = get_async_redis_client(
                self._redis_url, decode_responses=True, socket_timeout=None
            )
        return self._async_redis_client

    @property
    def binary_redis_client(self):
        if self._binary_redis_client is None:
            self._binary_redis_client = get_sync_redis_client(
                self._redis_url, decode_responses=False, socket_timeout=None
            )
        return self._binary_redis_client

    @property
    def async_binary_redis_client(self):
        if self._async_binary_redis_client is None:
            self._async_binary_redis_client = get_async_redis_client(
                self._redis_url, decode_responses=False, socket_timeout=None
            )
        return self._async_binary_redis_client

    def get_redis_client(self, asyncio: bool = False, binary: bool = False):
        if binary:
            return self.async_binary_redis_client if asyncio else self.binary_redis_client
        return self.async_redis_client if asyncio else self.redis_client

    def _batch_send_event_sync(self, prefixed_queue, messages: List[dict], pipe):
        lua_args = [self.redis_prefix.encode() if isinstance(self.redis_prefix, str) else self.redis_prefix]

        for message in messages:
            if 'data' in message:
                data = message['data'] if isinstance(message['data'], bytes) else dumps_str(message['data'])
            else:
                data = dumps_str(message)
            lua_args.append(data)

        client = self.get_redis_client(asyncio=False, binary=True)

        results = client.eval(
            LUA_SCRIPT_BATCH_SEND_EVENT,
            1,  
            prefixed_queue,  
            *lua_args  
        )

        return [r.decode('utf-8') if isinstance(r, bytes) else r for r in results]

    async def _batch_send_event(self, prefixed_queue, messages: List[dict], pipe):
        lua_args = [self.redis_prefix.encode() if isinstance(self.redis_prefix, str) else self.redis_prefix]

        for message in messages:
            if 'data' in message:
                data = message['data'] if isinstance(message['data'], bytes) else dumps_str(message['data'])
            else:
                data = dumps_str(message)
            lua_args.append(data)

        client = self.get_redis_client(asyncio=True, binary=True)

        results = await client.eval(
            LUA_SCRIPT_BATCH_SEND_EVENT,
            1,  
            prefixed_queue,  
            *lua_args  
        )

        return [r.decode('utf-8') if isinstance(r, bytes) else r for r in results]
    
    def is_urgent(self, routing_key):
        is_urgent = self.solo_urgent_retry.get(routing_key, False)
        if is_urgent == True:
            del self.solo_urgent_retry[routing_key]
        return is_urgent
    
    async def scan_priority_queues(self, base_queue: str) -> list:
        pattern = f"{self.redis_prefix}:QUEUE:{base_queue}:*"
        
        try:
            from jettask.messaging.registry import QueueRegistry
            registry = QueueRegistry(
                redis_client=self.redis_client,
                async_redis_client=self.async_redis_client,
                redis_prefix=self.redis_prefix
            )
            
            priority_queue_names = await registry.get_priority_queues_for_base(base_queue)
            priority_queues = set(priority_queue_names)
            
            if not priority_queues:
                all_queues = await registry.get_all_queues()
                for queue in all_queues:
                    if queue.startswith(f"{base_queue}:"):
                        priority_queues.add(queue)
            
            priority_queues.add(base_queue)
            
            sorted_queues = []
            for q in priority_queues:
                if ':' in q:
                    base, priority = q.rsplit(':', 1)
                    if base == base_queue and priority.isdigit():
                        sorted_queues.append((int(priority), q))
                    else:
                        sorted_queues.append((float('inf'), q))  
                else:
                    sorted_queues.append((float('inf'), q))  
            
            sorted_queues.sort(key=lambda x: x[0])
            return [q[1] for q in sorted_queues]
            
        except Exception as e:
            import traceback
            logger.error(f"Error scanning priority queues for {base_queue}: {e}\n{traceback.format_exc()}")
            return [base_queue]  
    
    async def _ensure_consumer_group_and_record_info(
        self,
        prefixed_queue: str,
        task_name: str,
        consumer_name: str,
        base_group_name: str
    ) -> str:
        actual_queue_name = prefixed_queue.replace(f"{self.redis_prefix}:QUEUE:", "")

        group_name = base_group_name

        try:
            await self.async_redis_client.xgroup_create(
                name=prefixed_queue,
                groupname=group_name,
                id="0",
                mkstream=True
            )
            logger.debug(f"Created consumer group {group_name} for queue {prefixed_queue}")
        except Exception as e:
            if "BUSYGROUP" in str(e):
                logger.debug(f"Consumer group {group_name} already exists for queue {prefixed_queue}")
            else:
                logger.warning(f"Error creating consumer group {group_name} for {prefixed_queue}: {e}")

        await self.record_group_info_async(
            actual_queue_name, task_name, group_name, consumer_name
        )

        return group_name

    async def get_priority_queues_direct(self, base_queue: str) -> list:
        from jettask.messaging.registry import QueueRegistry
        registry = QueueRegistry(self.redis_client, self.async_redis_client, self.redis_prefix)

        priority_queue_names = await registry.get_priority_queues_for_base(base_queue)
        priority_queues = []

        for pq_name in priority_queue_names:
            if ':' in pq_name and pq_name.rsplit(':', 1)[1].isdigit():
                prefixed_pq = f"{self.redis_prefix}:QUEUE:{pq_name}"
                priority = int(pq_name.rsplit(':', 1)[1])
                priority_queues.append((priority, prefixed_pq))

        priority_queues.sort(key=lambda x: x[0])

        base_prefixed = f"{self.redis_prefix}:QUEUE:{base_queue}"
        return [base_prefixed] + [q[1] for q in priority_queues]
  
    
    @classmethod
    def separate_by_key(cls, lst):
        groups = {}
        for item in lst:
            key = item[0]['routing_key']
            if key not in groups:
                groups[key] = []
            groups[key].append(item)
        result = []
        group_values = list(groups.values())
        while True:
            exists_data = False
            for values in group_values:
                try:
                    result.append(values.pop(0))
                    exists_data = True
                except:
                    pass
            if not exists_data:
                break
        return result
    
    async def _unified_task_checker(self, event_queue: asyncio.Queue, checker_type: str = 'solo_agg'):
        last_solo_running_state = defaultdict(dict)
        last_wait_time = defaultdict(int)
        queue_batch_tasks = defaultdict(list)
        left_queue_batch_tasks = defaultdict(list)
        
        delay_tasks = getattr(self, 'delay_tasks', []) if checker_type == 'delay' else []
        
        while True:
            has_work = False
            current_time = time.time()
            
            if checker_type == 'delay':
                put_count = 0
                need_del_index = []
                for i in range(len(delay_tasks)):
                    schedule_time = delay_tasks[i][0]
                    task = delay_tasks[i][1]
                    if schedule_time <= current_time:
                        try:
                            await self._async_put_task(event_queue, task)
                            need_del_index.append(i)
                            put_count += 1
                            has_work = True
                        except IndexError:
                            pass
                for i in need_del_index:
                    del delay_tasks[i]
                    
            elif checker_type == 'solo_agg':
                for queue in self.queues:
                    for agg_key, tasks in self.solo_agg_task[queue].items():
                        if not tasks:
                            continue
                            
                        has_work = True
                        need_del_index = []
                        need_lock_routing_keys = []
                        sort_by_tasks = self.separate_by_key(tasks)
                        max_wait_time = 5
                        max_records = 3
                        
                        for index, (routing, task) in enumerate(sort_by_tasks):
                            routing_key = routing['routing_key']
                            max_records = routing.get('max_records', 1)
                            max_wait_time = routing.get('max_wait_time', 0)
                            
                            with self.rlock:
                                if self.solo_running_state[queue].get(routing_key, 0) > 0:
                                    continue
                                    
                            if len(queue_batch_tasks[queue] + left_queue_batch_tasks[queue]) >= max_records:
                                break 
                                
                            task["routing"] = routing

                            if self.is_urgent(routing_key):
                                left_queue_batch_tasks[queue].append(task)
                            else:
                                queue_batch_tasks[queue].append(task)
                            need_lock_routing_keys.append(routing_key)
                            need_del_index.append(index)

                        for routing_key, count in Counter(need_lock_routing_keys).items():
                            with self.rlock:
                                self.solo_running_state[queue][routing_key] = count
                                
                        if last_solo_running_state[queue] != self.solo_running_state[queue]:
                            last_solo_running_state[queue] = self.solo_running_state[queue].copy()
                            
                        tasks = [task for index, task in enumerate(sort_by_tasks) if index not in need_del_index]
                        self.solo_agg_task[queue][agg_key] = tasks
                        
                        if (len(queue_batch_tasks[queue] + left_queue_batch_tasks[queue]) >= max_records or 
                            (last_wait_time[queue] and last_wait_time[queue] < current_time - max_wait_time)):
                            for task in queue_batch_tasks[queue]:
                                await self._async_put_task(event_queue, task)
                            for task in left_queue_batch_tasks[queue]:
                                await self._async_put_task(event_queue, task)    
                            queue_batch_tasks[queue] = []
                            left_queue_batch_tasks[queue] = []
                            last_wait_time[queue] = 0
                        elif last_wait_time[queue] == 0:
                            last_wait_time[queue] = current_time
            
            sleep_time = self._get_optimal_sleep_time(has_work, checker_type)
            await asyncio.sleep(sleep_time)
    
    def _get_optimal_sleep_time(self, has_work: bool, checker_type: str) -> float:
        if checker_type == 'delay':
            return 0.001 if has_work else 1.0
        elif has_work:
            return 0.001  
        else:
            return 0.01   
    
    
    async def async_check_solo_agg_tasks(self, event_queue: asyncio.Queue):
        await self._unified_task_checker(event_queue, checker_type='solo_agg')
    
    async def check_solo_agg_tasks(self, event_queue: asyncio.Queue):
        await self._unified_task_checker(event_queue, checker_type='solo_agg')
    
    def check_sole_tasks(self, event_queue: Union[deque, asyncio.Queue]):
        agg_task_mappings = {queue:  defaultdict(list) for queue in self.queues}
        agg_wait_task_mappings = {queue:  defaultdict(float) for queue in self.queues}
        task_max_wait_time_mapping = {}
        make_up_for_index_mappings = {queue:  defaultdict(int) for queue in self.queues} 
        while True:
            put_count = 0
            for queue in self.queues:
                agg_task = agg_task_mappings[queue]
                for routing_key, tasks in self.solo_routing_tasks[queue].items():
                    schedule_time = self.task_scheduler[queue][routing_key]
                    if tasks:
                        for task in tasks:
                            prev_routing = task[0]
                            if agg_key:= prev_routing.get('agg_key'):
                                if not self.running_task_state_mappings[queue][agg_key]:
                                    self.solo_running_state[queue][routing_key] = False
                                    break 
                    if (
                        schedule_time <= time.time()
                        and self.solo_running_state[queue][routing_key] == False
                    ) :
                            try:
                                routing, task = tasks.pop(0)
                            except IndexError:
                                continue
                            task["routing"] = routing
                            
                            agg_key = routing.get('agg_key')
                            if agg_key is not None:
                                start_time = agg_wait_task_mappings[queue][agg_key]
                                if not start_time:
                                    agg_wait_task_mappings[queue][agg_key] = time.time()
                                    start_time = agg_wait_task_mappings[queue][agg_key]
                                agg_task[agg_key].append(task)
                                max_wait_time = routing.get('max_wait_time', 3)
                                task_max_wait_time_mapping[agg_key] = max_wait_time
                                if len(agg_task[agg_key])>=routing.get('max_records', 100) or time.time()-start_time>=max_wait_time:
                                    logger.debug(f'{agg_key=} {len(agg_task[agg_key])} 已满，准备发车！{routing.get("max_records", 100)} {time.time()-start_time} {max_wait_time}')
                                    for task in agg_task[agg_key]:
                                        task['routing']['version'] = 1
                                        self.running_task_state_mappings[queue][agg_key][task['event_id']] = time.time()
                                        self._put_task(event_queue, task, urgent=self.is_urgent(routing_key))
                                    agg_task[agg_key] = []
                                    make_up_for_index_mappings[queue][agg_key] = 0 
                                    agg_wait_task_mappings[queue][agg_key] = 0
                            else:
                                self._put_task(event_queue, task, urgent=self.is_urgent(routing_key))
                            self.solo_running_state[queue][routing_key] = True
                            put_count += 1
                for agg_key in agg_task.keys():
                    if not agg_task[agg_key]:
                        continue
                    start_time = agg_wait_task_mappings[queue][agg_key]
                    max_wait_time = task_max_wait_time_mapping[agg_key]
                    if make_up_for_index_mappings[queue][agg_key]>= len(agg_task[agg_key])-1:
                        make_up_for_index_mappings[queue][agg_key] = 0
                    routing = agg_task[agg_key][make_up_for_index_mappings[queue][agg_key]]['routing']
                    routing_key = routing['routing_key']
                    self.solo_running_state[queue][routing_key] = False
                    make_up_for_index_mappings[queue][agg_key] += 1
                    if time.time()-start_time>=max_wait_time:
                        logger.debug(f'{agg_key=} {len(agg_task[agg_key])}被迫发车！ {time.time()-start_time} {max_wait_time}')
                        for task in agg_task[agg_key]:
                            task['routing']['version'] = 1
                            self.running_task_state_mappings[queue][agg_key][task['event_id']] = time.time()
                            self._put_task(event_queue, task, urgent=self.is_urgent(routing_key))
                        agg_task[agg_key] = []
                        make_up_for_index_mappings[queue][agg_key] = 0
                        agg_wait_task_mappings[queue][agg_key] = 0
            if not put_count:
                time.sleep(0.001)
            elif put_count < 5:
                time.sleep(0.0005)  
                
    async def check_batch_tasks(self, event_queue: asyncio.Queue):
        await asyncio.sleep(0.1)

    async def check_delay_tasks(self, event_queue: asyncio.Queue):
        await self._unified_task_checker(event_queue, checker_type='delay')

    def _handle_redis_error(self, error: Exception, consecutive_errors: int, queue: str = None) -> tuple[bool, int]:
        if isinstance(error, redis.exceptions.ConnectionError):
            logger.error(f'Redis连接错误: {error}')
            logger.error(traceback.format_exc())
            consecutive_errors += 1
            if consecutive_errors >= 5:
                logger.error(f'连续连接失败{consecutive_errors}次，重新创建连接')
                return True, 0
            return False, consecutive_errors
            
        elif isinstance(error, redis.exceptions.ResponseError):
            if "NOGROUP" in str(error) and queue:
                logger.warning(f'队列 {queue} 或消费者组不存在')
                return False, consecutive_errors
            else:
                logger.error(f'Redis错误: {error}')
                logger.error(traceback.format_exc())
                consecutive_errors += 1
                return False, consecutive_errors
        else:
            logger.error(f'意外错误: {error}')
            logger.error(traceback.format_exc())
            consecutive_errors += 1
            return False, consecutive_errors

    def _process_message_common(self, event_id: str, event_data: dict, queue: str, event_queue, is_async: bool = False, consumer_name: str = None, group_name: str = None):
        if event_id in self._claimed_message_ids:
            logger.debug(f"跳过已认领的消息 {event_id}")
            return event_id
        
        actual_event_id = event_id  
        parsed_event_data = None  
        
        if 'data' in event_data or b'data' in event_data:
            data_field = event_data.get('data') or event_data.get(b'data')
            if data_field:
                try:
                    if isinstance(data_field, bytes):
                        parsed_data = loads_str(data_field)
                    else:
                        parsed_data = data_field
                    if 'event_id' in parsed_data:
                        actual_event_id = parsed_data['event_id']
                    parsed_event_data = parsed_data
                except (ValueError, UnicodeDecodeError):
                    pass  
        
        final_event_data = parsed_event_data if parsed_event_data is not None else event_data
        
        routing = final_event_data.get("routing")
        
        actual_queue = final_event_data.get('queue', queue)

        if not group_name:
            prefixed_queue = self.get_prefixed_queue_name(queue)
            group_name = prefixed_queue

        offset = None
        if 'offset' in final_event_data:
            try:
                offset = int(final_event_data['offset'])
            except (ValueError, TypeError):
                pass
        elif 'offset' in event_data or b'offset' in event_data:
            offset_field = event_data.get('offset') or event_data.get(b'offset')
            if offset_field:
                try:
                    offset = int(offset_field)
                    final_event_data['offset'] = offset
                except (ValueError, TypeError):
                    pass

        task_item = {
            "queue": actual_queue,  
            "event_id": actual_event_id,
            "event_data": final_event_data,  
            "consumer": consumer_name,  
            "group_name": group_name,  
        }
        
        push_flag = True
        if routing:
            if agg_key := routing.get('agg_key'):
                self.solo_agg_task[queue][agg_key].append(
                    [routing, task_item]
                )
                push_flag = False
        
        if push_flag:
            if is_async:
                return ('async_put', task_item)
            else:
                self._put_task(event_queue, task_item)
        
        return event_id
    
    async def _execute_recovery_for_queue(self, queue: str, log_prefix: str = "Recovery") -> int:
        from jettask.utils.queue_matcher import find_matching_tasks
        task_names = find_matching_tasks(queue, self.app._tasks_by_queue, self.wildcard_mode)

        if not task_names:
            logger.debug(f"[{log_prefix}] No tasks found for queue {queue}")
            return 0

        total_recovered = 0

        for task_name in task_names:
            recovery_key = f"recovery_{task_name}"
            recovery = getattr(self, recovery_key, None)

            if not recovery:
                from jettask.worker.lifecycle import WorkerManager
                recovery = WorkerManager(
                    redis_client=self.redis_client,
                    async_redis_client=self.async_binary_redis_client,
                    redis_prefix=self.redis_prefix,
                    queue_formatter=lambda q: f"{self.redis_prefix}:QUEUE:{q}",
                    queue_registry=self.queue_registry,
                    app=self.app,
                    tasks=self.tasks,
                    task_event_queues=self.task_event_queues,
                    worker_id=self.worker_id
                )
                setattr(self, recovery_key, recovery)

            base_queue = queue.split(':')[0]

            try:
                current_consumer = self.get_consumer_name(base_queue)
            except Exception as e:
                logger.warning(f"[{log_prefix}] Failed to get consumer for queue {queue}: {e}")
                continue

            def get_event_queue_by_task(tn: str):
                if self.task_event_queues:
                    return self.task_event_queues.get(tn)
                return None

            try:
                recovered = await recovery.recover_offline_workers(
                    task_name=task_name,
                    event_queue=None,  
                    event_queue_callback=get_event_queue_by_task  
                )
                total_recovered += recovered

                if recovered > 0:
                    logger.debug(f"[{log_prefix}] Recovered {recovered} messages for task {task_name} on queue {queue}")
            except Exception as e:
                logger.error(f"[{log_prefix}] Error recovering task {task_name} on queue {queue}: {e}")
                continue

        return total_recovered

    async def handle_worker_offline_event(self, worker_id: str, queues: list = None):
        try:
            logger.debug(f"[Recovery Event] Received offline event for worker {worker_id}")
            
            if not queues:
                if self.app and hasattr(self.app, 'worker_state'):
                    worker_info = await self.app.worker_state.get_worker_info(worker_id)
                    if worker_info:
                        queues_str = worker_info.get('queues', '')
                        queues = queues_str.split(',') if queues_str else []
                else:
                    logger.warning(f"[Recovery Event] WorkerManager not available, cannot get worker info for {worker_id}")
                    return

            if not queues:
                logger.warning(f"[Recovery Event] No queues found for worker {worker_id}, skipping recovery")
                return

            if not self.task_event_queues:
                logger.warning(f"[Recovery Event] No task_event_queues available, recovered messages will not be executed")

            for queue in queues:
                if not queue.strip():
                    continue

                logger.debug(f"[Recovery Event] Triggering recovery for worker {worker_id} on queue {queue}")

                try:
                    recovered = await self._execute_recovery_for_queue(queue, log_prefix="Recovery Event")

                    if recovered > 0:
                        logger.debug(f"[Recovery Event] Recovered {recovered} messages from worker {worker_id} on queue {queue}")
                    else:
                        logger.debug(f"[Recovery Event] No messages to recover from worker {worker_id} on queue {queue}")
                except Exception as e:
                    logger.warning(f"[Recovery Event] Failed to recover queue {queue}: {e}")
                    continue

        except Exception as e:
            logger.error(f"[Recovery Event] Error handling offline event for worker {worker_id}: {e}", exc_info=True)

    async def _perform_self_recovery(self, queues: set, event_queue: dict):
        logger.debug("[Recovery Self] Starting self-recovery for current worker...")

        current_worker_id = self.worker_id

        if not current_worker_id:
            logger.debug("[Recovery Self] No worker_id available, skipping self-recovery")
            return

        worker_key = f"{self.redis_prefix}:WORKER:{current_worker_id}"
        logger.debug(f"[Recovery Self] Checking pending messages for worker: {current_worker_id}")

        def get_event_queue_by_task(task_name: str):
            if event_queue:
                return event_queue.get(task_name)
            return None

        total_recovered = 0

        for queue in queues:
            try:
                base_queue = queue
                if ':' in queue and queue.rsplit(':', 1)[-1].isdigit():
                    base_queue = queue.rsplit(':', 1)[0]

                current_consumer = None
                for _ in range(5):
                    try:
                        current_consumer = self.get_consumer_name(base_queue)
                        if current_consumer:
                            if base_queue != queue:
                                priority_suffix = queue.rsplit(':', 1)[-1]
                                current_consumer = f"{current_consumer}:{priority_suffix}"
                            break
                    except:
                        pass
                    await asyncio.sleep(0.1)

                if not current_consumer:
                    logger.warning(f"[Recovery Self] Cannot get consumer for queue {queue}, skipping")
                    continue

                stream_key = f"{self.redis_prefix}:QUEUE:{queue}"

                worker_data = await self.async_redis_client.hgetall(worker_key)
                if not worker_data:
                    logger.debug(f"[Recovery Self] Worker {current_worker_id} has no data")
                    continue

                decoded_worker_data = {}
                for k, v in worker_data.items():
                    key = k.decode('utf-8') if isinstance(k, bytes) else k
                    value = v.decode('utf-8') if isinstance(v, bytes) else v
                    decoded_worker_data[key] = value

                group_infos = []
                for key, value in decoded_worker_data.items():
                    if key.startswith('group_info:'):
                        try:
                            group_info = json.loads(value)
                            if group_info.get('queue') == base_queue:
                                group_infos.append(group_info)
                        except Exception as e:
                            logger.error(f"[Recovery Self] Error parsing group_info: {e}")

                if not group_infos:
                    logger.debug(f"[Recovery Self] No group_info for queue {queue}")
                    continue

                for group_info in group_infos:
                    try:
                        task_name = group_info.get('task_name')
                        group_name = group_info.get('group_name')

                        if not task_name or not group_name:
                            continue

                        offline_consumer_name = group_info.get('consumer_name')

                        pending_info = await self.async_binary_redis_client.xpending(stream_key, group_name)
                        if not pending_info or pending_info.get('pending', 0) == 0:
                            continue

                        detailed_pending = await self.async_binary_redis_client.xpending_range(
                            stream_key, group_name,
                            min='-', max='+', count=100,
                            consumername=offline_consumer_name
                        )

                        if not detailed_pending:
                            continue

                        logger.debug(
                            f"[Recovery Self] Found {len(detailed_pending)} pending messages "
                            f"for worker {current_worker_id}, queue {queue}, task {task_name}"
                        )

                        message_ids = [msg['message_id'] for msg in detailed_pending]
                        claimed_messages = await self.async_binary_redis_client.xclaim(
                            stream_key, group_name, current_consumer,
                            min_idle_time=0,  
                            message_ids=message_ids
                        )

                        if claimed_messages:
                            logger.debug(
                                f"[Recovery Self] Claimed {len(claimed_messages)} messages "
                                f"from {offline_consumer_name} to {current_consumer}"
                            )

                            task_event_queue = get_event_queue_by_task(task_name)
                            if task_event_queue:
                                for msg_id, msg_data in claimed_messages:
                                    if isinstance(msg_id, bytes):
                                        msg_id = msg_id.decode('utf-8')

                                    data_field = msg_data.get(b'data') or msg_data.get('data')
                                    if data_field:
                                        try:
                                            import msgpack
                                            parsed_data = msgpack.unpackb(data_field, raw=False)
                                            parsed_data['_task_name'] = task_name
                                            parsed_data['queue'] = queue

                                            task_item = {
                                                'queue': queue,
                                                'event_id': msg_id,
                                                'event_data': parsed_data,
                                                'consumer': current_consumer,
                                                'group_name': group_name
                                            }

                                            await task_event_queue.put(task_item)
                                            total_recovered += 1
                                        except Exception as e:
                                            logger.error(f"[Recovery Self] Error processing message: {e}")
                            else:
                                logger.warning(f"[Recovery Self] No event_queue for task {task_name}")

                    except Exception as e:
                        logger.error(f"[Recovery Self] Error recovering group {group_info}: {e}", exc_info=True)

            except Exception as e:
                logger.error(f"[Recovery Self] Error recovering queue {queue}: {e}", exc_info=True)

        if total_recovered > 0:
            logger.debug(f"[Recovery Self] Self-recovery completed: recovered {total_recovered} messages")
        else:
            logger.debug("[Recovery Self] Self-recovery completed: no pending messages found")

    async def _update_read_offset(self, queue: str, group_name: str, offset: int):
        try:
            if offset is None:
                return

            read_offset_key = f"{self.redis_prefix}:READ_OFFSETS"

            task_name = group_name.split(':')[-1]

            field = f"{queue}:{task_name}"

            await self.async_redis_client.eval(
                UPDATE_MAX_OFFSET_LUA,
                2,  
                read_offset_key,  
                field,  
                offset  
            )
            logger.debug(f"Updated read offset for {field}: {offset}")
        except Exception as e:
            logger.error(f"Error updating read offset: {e}")


    async def _initial_queue_discovery(self):
        if not self.wildcard_mode:
            return

        try:
            logger.debug("[QueueDiscovery] Performing initial queue discovery...")

            queue_members = await self.async_redis_client.smembers(
                self._queue_registry_key.encode()
            )

            discovered_queues = set()
            for queue_bytes in queue_members:
                queue_name = queue_bytes.decode('utf-8') if isinstance(queue_bytes, bytes) else str(queue_bytes)
                discovered_queues.add(queue_name)

            if not discovered_queues:
                logger.warning("[QueueDiscovery] QUEUE_REGISTRY is empty, initializing from existing data...")

                await self.queue_registry.initialize_from_existing_data()
                discovered_queues = await self.queue_registry.get_all_queues()

            logger.debug(f"[QueueDiscovery] Initial discovery found {len(discovered_queues)} queues: {discovered_queues}")

            self._discovered_queues = discovered_queues
            self.queues = [q for q in discovered_queues if q != '*']

            self.consumer_config['queues'] = self.queues

        except Exception as e:
            logger.error(f"[QueueDiscovery] Initial discovery failed: {e}", exc_info=True)
            self._discovered_queues = set()
            self.queues = []


    async def _check_queues_with_messages(
        self,
        all_queues: list,
        check_backlog: dict,
        group_name: str
    ) -> list:
        queues_with_messages = []

        try:
            queue_offsets_key = f"{self.redis_prefix}:QUEUE_OFFSETS"
            read_offsets_key = f"{self.redis_prefix}:READ_OFFSETS"

            pipe = self.async_redis_client.pipeline()

            for q in all_queues:
                actual_queue = q.replace(f"{self.redis_prefix}:QUEUE:", "")
                pipe.hget(queue_offsets_key, actual_queue)

            task_name = group_name.split(':')[-1]

            for q in all_queues:
                actual_queue = q.replace(f"{self.redis_prefix}:QUEUE:", "")
                field = f"{actual_queue}:{task_name}"
                pipe.hget(read_offsets_key, field)

            results = await pipe.execute()

            half_len = len(all_queues)
            for i, q in enumerate(all_queues):
                if check_backlog.get(q, True):
                    queues_with_messages.append(q)
                    logger.debug(f"Queue {q} needs to read pending messages, skipping offset check")
                    continue

                sent_offset = results[i]  
                read_offset = results[half_len + i]  

                sent = int(sent_offset) if sent_offset else 0
                read = int(read_offset) if read_offset else 0

                if sent > read:
                    queues_with_messages.append(q)
                    logger.debug(f"Queue {q} has {sent - read} unread messages (sent={sent}, read={read})")

            if not queues_with_messages:
                logger.debug("No queues have unread messages, will wait for new messages")

        except Exception as e:
            logger.debug(f"Failed to check queue offsets: {e}")
            queues_with_messages = all_queues

        return queues_with_messages

    async def _ensure_group_info_for_all_queues(
        self,
        all_queues: list,
        task_name: str,
        consumer_name: str,
        group_name: str
    ):
        for queue in all_queues:
            await self._ensure_consumer_group_and_record_info(
                queue, task_name, consumer_name, group_name
            )

        logger.debug(f"Restored group_info for {len(all_queues)} queues for task {task_name}")

    async def _initialize_queue_for_task(
        self,
        queue_name: str,
        task_name: str,
        consumer_name: str,
        lastid: dict,
        check_backlog: dict,
        base_group_name: str
    ):
        await self._ensure_consumer_group_and_record_info(
            queue_name, task_name, consumer_name, base_group_name
        )

        lastid[queue_name] = "0"
        check_backlog[queue_name] = True

        logger.debug(f"Initialized queue {queue_name} for task {task_name}")

    async def _initialize_all_queues_for_task(
        self,
        all_queues: list,
        task_name: str,
        consumer_name: str,
        lastid: dict,
        check_backlog: dict,
        base_group_name: str
    ) -> list:
        for queue in all_queues:
            await self._initialize_queue_for_task(
                queue, task_name, consumer_name, lastid, check_backlog, base_group_name
            )

        logger.debug(f"Initialized {len(all_queues)} queues for task {task_name}")

        return all_queues

    async def _read_messages_from_queues(
        self,
        all_queues: list,
        check_backlog: dict,
        group_name: str,
        lastid: dict,
        task_name: str,
        messages_needed: int
    ) -> list:
        messages = []

        queues_with_messages = await self._check_queues_with_messages(
            all_queues, check_backlog, group_name
        )

        if not queues_with_messages:
            return messages

        for q in queues_with_messages:
            if messages_needed <= 0:
                break  

            q_bytes = q.encode() if isinstance(q, str) else q
            if check_backlog.get(q, True):
                myid = lastid.get(q, "0-0")
            else:
                myid = ">"
            myid_bytes = myid.encode() if isinstance(myid, str) else myid

            try:
                q_messages = await self.async_binary_redis_client.xreadgroup(
                    groupname=group_name,
                    consumername=self.worker_id,
                    streams={q_bytes: myid_bytes},
                    count=messages_needed,  
                    block=100  
                )
                logger.debug(f'{group_name=} {q_bytes=} {self.worker_id=} {q_messages=}')
                if q_messages:
                    messages.extend(q_messages)
                    messages_read = len(q_messages[0][1]) if q_messages else 0
                    messages_needed -= messages_read

                    if messages_read > 0 and messages_needed > 0:
                        break  

            except Exception as e:
                if "NOGROUP" in str(e):
                    logger.warning(f"NOGROUP error for queue {q}, recreating consumer group...")
                    try:
                        await self._initialize_queue_for_task(
                            q, task_name, self.worker_id, lastid, check_backlog, base_group_name=group_name
                        )

                        if q not in all_queues:
                            all_queues.append(q)
                            logger.debug(f"Re-added queue {q} to all_queues after NOGROUP recovery")
                    except Exception as recreate_error:
                        logger.error(f"Failed to recreate consumer group for {q}: {recreate_error}")
                else:
                    logger.debug(f"Error reading from queue {q}: {e}")
                continue

        return messages

    async def _report_delivered_offsets(self, messages: list, group_name: str):
        try:
            from jettask.utils.stream_backlog import report_delivered_offset
            for msg in messages:
                stream_name = msg[0]
                if isinstance(stream_name, bytes):
                    stream_name = stream_name.decode('utf-8')
                queue_name = stream_name.replace(f"{self.redis_prefix}:STREAM:", "")
                await report_delivered_offset(
                    self.async_redis_client,
                    self.redis_prefix,
                    queue_name,
                    group_name,
                    [msg]
                )
        except Exception as e:
            logger.debug(f"Failed to report delivered offset: {e}")

    async def listen_event_by_task(self, task_obj: "Task", queue_name: str, prefetch_multiplier: int):
        check_backlog = {}  
        lastid = {}  
        consecutive_errors = 0
        max_consecutive_errors = 5

        task_name = task_obj.name

        task_event_queue = self.task_event_queues.get(task_name)
        if not task_event_queue:
            logger.error(f"No event queue found for task {task_name}")
            return


        all_queues = []
        prefixed_queue = queue_name 

        group_name = f"{prefixed_queue}:{task_name}"


        last_priority_update = 0

        while not self._stop_reading:
            current_time = time.time()
            if current_time - last_priority_update >= 1:  
                new_all_queues = await self.get_priority_queues_direct(queue_name)
                if new_all_queues != all_queues:

                    new_queues = set(new_all_queues) - set(all_queues)
                    if new_queues:
                        for new_queue in new_queues:
                            await self._initialize_queue_for_task(
                                new_queue, task_name, self.worker_id, lastid, check_backlog, group_name
                            )

                    all_queues = new_all_queues
                last_priority_update = current_time


            current_queue_size = task_event_queue.qsize() if hasattr(task_event_queue, 'qsize') else 0
            if current_queue_size >= prefetch_multiplier:
                await asyncio.sleep(0.01)  
                continue

            messages_needed = prefetch_multiplier - current_queue_size

            if messages_needed <= 0:
                await asyncio.sleep(0.01)
                continue

            messages = await self._read_messages_from_queues(
                all_queues, check_backlog, group_name, lastid, task_name, messages_needed
            )

            if not messages:
                await asyncio.sleep(0.2)
                continue


            try:
                consecutive_errors = 0

                await self._report_delivered_offsets(messages, group_name)
                
                skip_message_ids = []
                
                max_offsets_per_queue = {}
                
                for message in messages:
                    stream_name = message[0]
                    if isinstance(stream_name, bytes):
                        stream_name = stream_name.decode('utf-8')
                    
                    if len(message[1]) == 0:
                        check_backlog[stream_name] = False
                    
                    for event in message[1]:
                        event_id = event[0]
                        lastid[stream_name] = event_id
                        if isinstance(event_id, bytes):
                            event_id = event_id.decode('utf-8')
                        event_data = event[1]
                        
                        should_process = True
                        
                        try:
                            if b'data' in event_data or 'data' in event_data:
                                data_field = event_data.get(b'data') or event_data.get('data')
                                
                                parsed_data = loads_str(data_field)

                            
                                target_tasks = parsed_data.get('_target_tasks', None)
                                if target_tasks and task_name not in target_tasks:
                                    should_process = False
                                
                                if should_process:
                                    parsed_data['_task_name'] = task_name
                                    
                                    offset_field = event_data.get(b'offset') or event_data.get('offset')
                                    message_offset = None
                                    if offset_field:
                                        if isinstance(offset_field, bytes):
                                            offset_field = offset_field.decode('utf-8')
                                        parsed_data['offset'] = offset_field
                                        try:
                                            message_offset = int(offset_field)
                                        except (ValueError, TypeError):
                                            pass
                                    
                                    event_data.clear()
                                    for key, value in parsed_data.items():
                                        event_data[key] = value
                                    
                                    if message_offset is not None:
                                        actual_queue_name = stream_name.replace(f"{self.redis_prefix}:QUEUE:", "")
                                        if actual_queue_name not in max_offsets_per_queue:
                                            max_offsets_per_queue[actual_queue_name] = message_offset
                                        else:
                                            max_offsets_per_queue[actual_queue_name] = max(max_offsets_per_queue[actual_queue_name], message_offset)
                                    
                                    logger.debug(f"Task {task_name} will process message {event_id}")
                            else:
                                should_process = False
                        except Exception as e:
                            logger.error(f"Task {task_name}: Error parsing message data: {e}")
                        
                        if should_process:
                            actual_queue = event_data.get('queue', queue_name)

                            result = self._process_message_common(
                                event_id, event_data, actual_queue, task_event_queue,
                                is_async=True, consumer_name=self.worker_id, group_name=group_name
                            )
                            if isinstance(result, tuple) and result[0] == 'async_put':
                                await self._async_put_task(task_event_queue, result[1])
                                logger.debug(f"Put task {event_id} into task_event_queue")
                        else:
                            skip_message_ids.append(event_id)
                        
                
                if skip_message_ids:
                    group_name_bytes = group_name.encode() if isinstance(group_name, str) else group_name
                    for q in all_queues:
                        q_bytes = q.encode() if isinstance(q, str) else q
                        try:
                            await self.async_binary_redis_client.xack(q_bytes, group_name_bytes, *skip_message_ids)
                        except:
                            pass  
                    logger.debug(f"Task {task_name} batch ACKed {len(skip_message_ids)} skipped messages")

                if max_offsets_per_queue:
                    for queue_name, max_offset in max_offsets_per_queue.items():
                        asyncio.create_task(self._update_read_offset(queue_name, group_name, max_offset))
                    logger.debug(f"Updated read offsets for {len(max_offsets_per_queue)} queues")
                    
            except Exception as e:
                error_msg = str(e)
                import traceback
                traceback.print_exc()
                logger.error(f"Error in task listener {task_name}: {e}")
                consecutive_errors += 1
                if consecutive_errors >= max_consecutive_errors:
                    logger.error(f"Too many errors for task {task_name}, restarting...")
                    consecutive_errors = 0
                await asyncio.sleep(min(consecutive_errors, 5))


    async def _listen_task(self, task_obj: "Task", prefetch_multiplier: int):
        if task_obj.is_wildcard_queue:
            await self._listen_task_with_wildcard(task_obj, prefetch_multiplier)
        else:
            await self._start_single_queue_listener(task_obj, task_obj.queue, prefetch_multiplier)

    async def _start_single_queue_listener(self, task_obj: "Task", queue_name: str, prefetch_multiplier: int):
        task_name = task_obj.name

        await self._register_queue_for_task(queue_name, task_name)

        logger.debug(f"[Task: {task_name}] 开始监听队列: {queue_name}")

        await self.listen_event_by_task(
            task_obj,
            queue_name,
            prefetch_multiplier
        )

    async def _listen_task_with_wildcard(self, task_obj: "Task", prefetch_multiplier: int):
        task_name = task_obj.name
        wildcard_pattern = task_obj.queue

        logger.debug(f"[Task: {task_name}] 开始监听通配符队列: {wildcard_pattern}")

        monitored_queues = set()

        discovery_interval = 5.0  

        while not self._stop_reading:
            try:
                matched_queues = await self.queue_registry.discover_matching_queues(wildcard_pattern)

                new_queues = matched_queues - monitored_queues

                if new_queues:
                    logger.debug(f"[Task: {task_name}] 发现新队列: {list(new_queues)}")

                    for queue_name in new_queues:
                        listen_task = asyncio.create_task(
                            self._start_single_queue_listener(
                                task_obj,
                                queue_name,
                                prefetch_multiplier
                            )
                        )
                        self._background_tasks.append(listen_task)
                        monitored_queues.add(queue_name)

                        logger.debug(f"[Task: {task_name}] 已为队列 {queue_name} 启动监听")

                await asyncio.sleep(discovery_interval)

            except asyncio.CancelledError:
                logger.debug(f"[Task: {task_name}] 通配符队列监听已取消")
                break
            except Exception as e:
                logger.error(f"[Task: {task_name}] 通配符队列监听出错: {e}", exc_info=True)
                await asyncio.sleep(discovery_interval)

    async def _register_queue_for_task(self, queue: str, task_name: str):
        if queue not in self.queues:
            self.queues.append(queue)
            self.queues.sort()

            self._queue_stop_flags[queue] = False

            try:
                if self.worker_id and self.app and hasattr(self.app, 'worker_state'):
                    await self.app.worker_state.update_worker_field(
                        self.worker_id,
                        'queues',
                        ','.join(sorted(self.queues))
                    )
                    logger.debug(f"[Task: {task_name}] 新增队列 {queue}，当前总队列: {self.queues}")
            except Exception as e:
                logger.error(f"[Task: {task_name}] 更新 worker queues 字段失败: {e}", exc_info=True)


    async def listening_event(self, prefetch_multiplier: int = 1):
        if not self.task_event_queues:
            raise RuntimeError("task_event_queues not initialized")

        if not isinstance(self.task_event_queues, dict):
            raise TypeError(f"task_event_queues must be a dict[str, asyncio.Queue], got {type(self.task_event_queues)}")


        self._background_tasks = []

        if not self.tasks:
            raise RuntimeError("No tasks configured for EventPool")

        tasks = []

        if not self.worker_id:
            raise RuntimeError("Worker ID not initialized, cannot start listeners")

        for task_name, task_obj in self.tasks.items():
            if not task_obj:
                logger.warning(f"Task {task_name} has no task object, skipping...")
                continue

            if not task_obj.queue:
                logger.warning(f"Task {task_name} has no queue configured, skipping...")
                continue

            logger.debug(f"为任务 {task_name} 启动监听，队列模式: {task_obj.queue}, worker_id: {self.worker_id}")

            task = asyncio.create_task(
                self._listen_task(
                    task_obj=task_obj,
                    prefetch_multiplier=prefetch_multiplier
                )
            )
            tasks.append(task)
            self._background_tasks.append(task)



        recovery_task = asyncio.create_task(
            self.offline_recovery.start(recovery_interval=30.0)
        )
        self._background_tasks.append(recovery_task)
        logger.debug("[Recovery] Started offline worker recovery loop")

        cleanup_task = asyncio.create_task(
            self.offline_recovery.start_cleanup_task(cleanup_interval=86400.0)
        )
        self._background_tasks.append(cleanup_task)
        logger.debug("[Cleanup] Started offline worker cleanup task")

        try:
            await asyncio.gather(*tasks)
        except asyncio.CancelledError:
            logger.debug("listening_event tasks cancelled, cleaning up...")

            for task in self._background_tasks:
                if not task.done():
                    task.cancel()
            if self._background_tasks:
                try:
                    await asyncio.wait_for(
                        asyncio.gather(*self._background_tasks, return_exceptions=True),
                        timeout=0.2
                    )
                except asyncio.TimeoutError:
                    logger.debug("Some background tasks did not complete in time")
            raise

    def read_pending(self, groupname: str, queue: str, asyncio: bool = False):
        client = self.get_redis_client(asyncio, binary=True)
        prefixed_queue = self.get_prefixed_queue_name(queue)
        return client.xpending(prefixed_queue, groupname)

    def ack(self, queue, event_id, asyncio: bool = False):
        client = self.get_redis_client(asyncio, binary=True)
        prefixed_queue = self.get_prefixed_queue_name(queue)
        result = client.xack(prefixed_queue, prefixed_queue, event_id)
        if event_id in self._claimed_message_ids:
            self._claimed_message_ids.remove(event_id)
        return result
    def _safe_redis_operation(self, operation, *args, max_retries=3, **kwargs):
        for attempt in range(max_retries):
            try:
                return operation(*args, **kwargs)
            except (redis.exceptions.TimeoutError, redis.exceptions.ConnectionError) as e:
                if attempt == max_retries - 1:
                    logger.error(f"Redis操作失败，已重试{max_retries}次: {e}")
                    raise

                logger.warning(f"Redis操作失败，第{attempt + 1}次重试: {e}")
                time.sleep(min(2 ** attempt, 5))  
    
    def cleanup(self):
        self._stop_reading = True

        if hasattr(self, 'offline_recovery') and self.offline_recovery:
            if hasattr(self.offline_recovery, 'stop_recovery'):
                self.offline_recovery.stop_recovery()
            if hasattr(self.offline_recovery, 'stop_cleanup'):
                self.offline_recovery.stop_cleanup()

        logger.debug("EventPool cleanup completed")


"""
认证提供者抽象接口

定义了认证系统的标准接口，支持多种认证方式的实现：
- 本地认证（基于配置的用户名密码）
- 远程API认证（调用企业现有的认证服务）
- LDAP认证
- OAuth2认证
- 自定义认证
"""
import logging
from abc import ABC, abstractmethod
from typing import Optional, Dict, Any
from dataclasses import dataclass

logger = logging.getLogger(__name__)


@dataclass
class AuthResult:
    """认证结果"""
    success: bool
    username: Optional[str] = None
    user_info: Optional[Dict[str, Any]] = None  
    error_message: Optional[str] = None


class AuthProvider(ABC):
    """
    认证提供者抽象基类

    所有认证实现都需要继承此类并实现 authenticate 方法
    """

    @abstractmethod
    async def verify_token(self, token: str) -> AuthResult:
        pass

    @abstractmethod
    def get_provider_name(self) -> str:
        pass

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__}(provider={self.get_provider_name()})>"



"""
任务状态和结果定义
"""
from enum import Enum
from dataclasses import dataclass
from typing import Any, Optional, Dict
import time


class TaskType(Enum):
    """任务类型"""
    ONCE = "once"        
    INTERVAL = "interval"  
    CRON = "cron"        


class TaskStatus(Enum):
    """任务状态枚举"""
    PENDING = "pending"          
    RUNNING = "running"          
    SUCCESS = "success"          
    ERROR = "error"              
    FAILED = "failed"            
    
    DELAYED = "delayed"          
    REJECTED = "rejected"        
    RETRY = "retry"              
    TIMEOUT = "timeout"          
    CANCELLED = "cancelled"      
    
    SCHEDULED = "scheduled"      
    
    @classmethod
    def is_terminal(cls, status: 'TaskStatus') -> bool:
        return status in {cls.SUCCESS, cls.ERROR, cls.FAILED, cls.REJECTED, cls.TIMEOUT, cls.CANCELLED}
    
    @classmethod
    def is_active(cls, status: 'TaskStatus') -> bool:
        return status in {cls.PENDING, cls.RUNNING, cls.DELAYED, cls.RETRY, cls.SCHEDULED}


@dataclass
class TaskResult:
    """
    任务结果对象
    apply_async 返回的结果，包含任务的完整信息
    """
    id: str                                    
    name: str                                   
    queue: str                                  
    status: TaskStatus = TaskStatus.PENDING    
    created_at: float = None                   
    trigger_time: float = None                 
    scheduled_task_id: Optional[int] = None    
    args: tuple = None                         
    kwargs: dict = None                        
    metadata: Dict[str, Any] = None           
    
    def __post_init__(self):
        if self.created_at is None:
            self.created_at = time.time()
        if self.trigger_time is None:
            self.trigger_time = self.created_at
        if self.args is None:
            self.args = ()
        if self.kwargs is None:
            self.kwargs = {}
        if self.metadata is None:
            self.metadata = {}
    
    def __str__(self) -> str:
        return f"TaskResult(id={self.id}, name={self.name}, status={self.status.value})"
    
    def __repr__(self) -> str:
        return (f"TaskResult(id={self.id}, name={self.name}, queue={self.queue}, "
                f"status={self.status.value}, created_at={self.created_at})")
    
    async def wait(self, timeout: Optional[float] = None):
        raise NotImplementedError("wait method not implemented yet")
    
    async def get_result(self, timeout: Optional[float] = None):
        raise NotImplementedError("get_result method not implemented yet")
    
    async def cancel(self) -> bool:
        raise NotImplementedError("cancel method not implemented yet")
    
    async def get_status(self) -> TaskStatus:
        raise NotImplementedError("get_status method not implemented yet")
    
    @property
    def is_ready(self) -> bool:
        return TaskStatus.is_terminal(self.status)
    
    @property
    def is_successful(self) -> bool:
        return self.status == TaskStatus.SUCCESS
    
    @property
    def is_failed(self) -> bool:
        return self.status in {TaskStatus.ERROR, TaskStatus.TIMEOUT, TaskStatus.REJECTED}


"""
统一的 PostgreSQL 消费者管理器
基于 NamespaceWorkerManagerBase 实现
"""
import asyncio
import logging
import multiprocessing
from typing import Dict, Optional
from jettask.core.namespace_manager_base import NamespaceWorkerManagerBase
from jettask.core.namespace import NamespaceContext
from jettask.persistence.consumer import PostgreSQLConsumer

logger = logging.getLogger(__name__)


class UnifiedConsumerManager(NamespaceWorkerManagerBase):
    """
    统一的消费者管理器

    基于 NamespaceWorkerManagerBase，专注于消费者特定的逻辑
    """

    def __init__(self,
                 task_center_url: str,
                 check_interval: int = 30,
                 concurrency: int = 4,
                 prefetch_multiplier: int = 100,
                 api_key: str = None,
                 debug: bool = False):
        self.concurrency = concurrency
        self.prefetch_multiplier = prefetch_multiplier

        self.consumer_instance: Optional[PostgreSQLConsumer] = None  
        self.consumer_processes: Dict[str, multiprocessing.Process] = {}  

        super().__init__(task_center_url, check_interval, debug, api_key=api_key)

    async def _run_worker_for_namespace(self, ns: NamespaceContext):
        logger.info(f"启动命名空间 {ns.name} 的消费者")

        try:
            logger.info(f"命名空间 {ns.name} 配置:")
            logger.info(f"  - Redis: 已配置")
            logger.info(f"  - PostgreSQL: 已配置")
            logger.info(f"  - Redis Prefix: {ns.redis_prefix}")

            self.consumer_instance = PostgreSQLConsumer(
                pg_config=ns.pg_config,
                redis_config=ns.redis_config,
                prefix=ns.redis_prefix,
                namespace_name=ns.name
            )

            logger.info(f"✓ 消费者已创建: {ns.name}")

            await self.consumer_instance.start(
                concurrency=self.concurrency,
                prefetch_multiplier=self.prefetch_multiplier
            )

        except Exception as e:
            logger.error(f"消费者运行失败: {e}", exc_info=self.debug)
            raise
        finally:
            if self.consumer_instance:
                await self.consumer_instance.stop()
                logger.info(f"消费者已停止: {ns.name}")

    def _should_start_worker(self, ns: NamespaceContext) -> bool:
        return ns.has_pg and ns.has_redis

    async def _start_worker(self, ns_or_name):
        ns_name = ns_or_name.name if isinstance(ns_or_name, NamespaceContext) else ns_or_name

        if ns_name in self.consumer_processes:
            process = self.consumer_processes[ns_name]
            if process.is_alive():
                logger.debug(f"命名空间 {ns_name} 的消费者进程已在运行")
                return
            else:
                logger.info(f"清理已停止的消费者进程: {ns_name}")
                process.terminate()
                process.join(timeout=5)

        logger.info(f"启动命名空间 {ns_name} 的消费者进程")

        base_url = self._get_base_url()
        single_ns_url = f"{base_url}/api/task/v1/{ns_name}"

        process = multiprocessing.Process(
            target=_run_consumer_in_process,
            args=(single_ns_url, self.concurrency, self.prefetch_multiplier, self.api_key, self.debug),
            name=f"Consumer-{ns_name}"
        )
        process.start()
        self.consumer_processes[ns_name] = process

        logger.info(f"✓ 消费者进程已启动: {ns_name} (PID: {process.pid})")

    async def _stop_worker(self, ns_name: str):
        if ns_name in self.consumer_processes:
            process = self.consumer_processes[ns_name]
            logger.info(f"停止消费者进程: {ns_name}")
            process.terminate()
            process.join(timeout=10)
            if process.is_alive():
                logger.warning(f"强制终止消费者进程: {ns_name}")
                process.kill()
                process.join()
            del self.consumer_processes[ns_name]

    def _get_running_workers(self) -> set[str]:
        return set(self.consumer_processes.keys())

    async def _namespace_check_loop(self):
        logger.info("命名空间检查循环已启动（包含健康检查）")

        while self.running:
            try:
                dead_processes = []
                for ns_name, process in self.consumer_processes.items():
                    if not process.is_alive():
                        logger.warning(f"消费者进程 {ns_name} 已停止 (退出码: {process.exitcode})")
                        dead_processes.append(ns_name)

                for ns_name in dead_processes:
                    logger.info(f"重启消费者进程: {ns_name}")
                    await self._start_worker(ns_name)

                await super()._namespace_check_loop()
                return  

            except Exception as e:
                logger.error(f"健康检查循环异常: {e}", exc_info=self.debug)
                await asyncio.sleep(10)


def _run_consumer_in_process(task_center_url: str, concurrency: int,
                             prefetch_multiplier: int, api_key: str, debug: bool):
    import logging
    import re

    match = re.search(r'/api/task/v1/([^/]+)/?$', task_center_url)
    namespace_name = match.group(1) if match else 'unknown'

    log_level = logging.DEBUG if debug else logging.INFO
    logging.basicConfig(
        level=log_level,
        format=f'%(asctime)s - [{namespace_name}] - %(name)s - %(levelname)s - %(message)s'
    )

    logger = logging.getLogger(__name__)

    manager = UnifiedConsumerManager(
        task_center_url=task_center_url,
        concurrency=concurrency,
        prefetch_multiplier=prefetch_multiplier,
        api_key=api_key,
        debug=debug
    )

    try:
        logger.info(f"消费者进程启动: {namespace_name} (PID: {multiprocessing.current_process().pid})")
        asyncio.run(manager.run())
    except KeyboardInterrupt:
        logger.info("进程收到中断信号")
    except Exception as e:
        logger.error(f"进程异常退出: {e}", exc_info=debug)
        raise


__all__ = ['UnifiedConsumerManager']



"""
过滤框架层堆栈信息的工具函数
"""
import traceback
import sys
from typing import Optional


def filter_framework_traceback(exc_type=None, exc_value=None, exc_traceback=None) -> str:
    if exc_type is None:
        exc_type, exc_value, exc_traceback = sys.exc_info()
    
    if exc_traceback is None:
        return "No traceback available"
    
    framework_paths = [
        '/jettask/executors/',
        '/jettask/core/',
        '/jettask/monitoring/',
        '/jettask/utils/',
        'asyncio/',
        'concurrent/',
        'multiprocessing/',
    ]
    
    tb_frames = []
    tb = exc_traceback
    
    business_frames = []
    while tb is not None:
        frame = tb.tb_frame
        filename = frame.f_code.co_filename
        
        is_framework = any(path in filename for path in framework_paths)
        
        if not is_framework:
            business_frames.append(tb)
        
        tb = tb.tb_next
    
    if business_frames:
        lines = []
        for tb in business_frames:
            frame = tb.tb_frame
            filename = frame.f_code.co_filename
            lineno = tb.tb_lineno
            name = frame.f_code.co_name
            
            lines.append(f'  File "{filename}", line {lineno}, in {name}\n')
            
            try:
                import linecache
                line = linecache.getline(filename, lineno, frame.f_globals)
                if line:
                    lines.append(f'    {line.strip()}\n')
            except:
                pass
        
        if lines:
            result = "Traceback (most recent call last):\n"
            result += "".join(lines)
            result += f"{exc_type.__name__}: {exc_value}"
            return result
    
    return f"{exc_type.__name__}: {exc_value}"


def get_clean_exception_message() -> str:
    exc_type, exc_value, exc_traceback = sys.exc_info()
    
    if exc_type is None:
        return "No exception"
    
    if exc_value:
        return f"{exc_type.__name__}: {exc_value}"
    
    return f"{exc_type.__name__}"


"""
Task Router for modular task organization
Similar to FastAPI's APIRouter
"""

from typing import Dict, List, Any, Callable
import logging
from jettask.utils.rate_limit.config import RateLimitConfig

logger = logging.getLogger(__name__)


class TaskRouter:
    """
    任务路由器，用于模块化组织任务
    
    使用示例：
        # tasks/email_tasks.py
        from jettask import TaskRouter
        
        router = TaskRouter(prefix="email", queue="emails")
        
        @router.task()
        async def send_welcome_email(user_id: str):
            # 实际任务名会是: email.send_welcome_email
            # 默认队列会是: emails
            pass
        
        # main.py
        from jettask import Jettask
        from tasks.email_tasks import router as email_router
        
        app = Jettask(redis_url="redis://localhost:6379/0")
        app.register_router(email_router)
    """
    
    def __init__(
        self,
        prefix: str = None,
        queue: str = None,
        tags: List[str] = None,
        default_timeout: int = None,
        default_max_retries: int = None,
        default_retry_delay: int = None,
    ):
        self.prefix = prefix
        self.default_queue = queue
        self.tags = tags or []
        self.default_timeout = default_timeout
        self.default_max_retries = default_max_retries
        self.default_retry_delay = default_retry_delay
        
        self._tasks: Dict[str, Dict[str, Any]] = {}
        
    def task(
        self,
        name: str = None,
        queue: str = None,
        timeout: int = None,
        max_retries: int = None,
        retry_delay: int = None,
        rate_limit: RateLimitConfig = None,
        auto_ack: bool = True,
        **kwargs
    ):
        def decorator(func: Callable):
            task_name = name or func.__name__
            if self.prefix:
                full_task_name = f"{self.prefix}.{task_name}"
            else:
                full_task_name = task_name

            task_config = {
                'func': func,
                'name': full_task_name,
                'queue': queue or self.default_queue,
                'timeout': timeout or self.default_timeout,
                'max_retries': max_retries or self.default_max_retries,
                'retry_delay': retry_delay or self.default_retry_delay,
                'rate_limit': rate_limit,
                'auto_ack': auto_ack,
                'tags': self.tags,
                **kwargs
            }

            task_config = {k: v for k, v in task_config.items() if v is not None}

            self._tasks[full_task_name] = task_config

            return func

        return decorator
    
    def include_router(self, router: 'TaskRouter', prefix: str = None):
        if prefix:
            if self.prefix:
                combined_prefix = f"{self.prefix}.{prefix}"
            else:
                combined_prefix = prefix
        else:
            combined_prefix = self.prefix
        
        for task_name, task_config in router._tasks.items():
            if combined_prefix and not task_name.startswith(combined_prefix):
                if router.prefix and task_name.startswith(router.prefix):
                    new_name = task_name.replace(router.prefix, combined_prefix, 1)
                else:
                    new_name = f"{combined_prefix}.{task_name}"
            else:
                new_name = task_name
            
            new_config = task_config.copy()
            new_config['name'] = new_name
            
            if 'queue' not in new_config or new_config['queue'] is None:
                new_config['queue'] = self.default_queue
            
            self._tasks[new_name] = new_config
    
    def get_tasks(self) -> Dict[str, Dict[str, Any]]:
        return self._tasks


"""
API 响应模型定义
包含所有 API 接口的响应模型和示例数据
"""
from typing import Optional, List, Dict, Any
from datetime import datetime
from pydantic import BaseModel, Field, ConfigDict



class SuccessResponse(BaseModel):
    """通用成功响应"""
    success: bool = Field(default=True, description="请求是否成功")
    message: Optional[str] = Field(None, description="响应消息")

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "success": True,
            "message": "操作成功"
        }
    })


class DataResponse(BaseModel):
    """通用数据响应"""
    success: bool = Field(default=True, description="请求是否成功")
    data: Any = Field(..., description="响应数据")
    message: Optional[str] = Field(None, description="响应消息")

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "success": True,
            "data": {},
            "message": "获取成功"
        }
    })


class PaginatedDataResponse(BaseModel):
    """分页数据响应"""
    success: bool = Field(default=True, description="请求是否成功")
    data: List[Any] = Field(default=[], description="数据列表")
    total: int = Field(..., description="总记录数", ge=0)
    page: int = Field(..., description="当前页码", ge=1)
    page_size: int = Field(..., description="每页大小", ge=1, le=100)

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "success": True,
            "data": [],
            "total": 0,
            "page": 1,
            "page_size": 20
        }
    })



class SystemStatsData(BaseModel):
    """系统统计数据"""
    total_queues: int = Field(..., description="队列总数", ge=0)
    total_tasks: int = Field(..., description="任务总数", ge=0)
    active_workers: int = Field(..., description="活跃 Worker 数", ge=0)
    pending_tasks: int = Field(..., description="待处理任务数", ge=0)
    processing_tasks: int = Field(..., description="处理中任务数", ge=0)
    completed_tasks: int = Field(..., description="已完成任务数", ge=0)
    failed_tasks: int = Field(..., description="失败任务数", ge=0)

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "total_queues": 5,
            "total_tasks": 1250,
            "active_workers": 12,
            "pending_tasks": 45,
            "processing_tasks": 8,
            "completed_tasks": 1180,
            "failed_tasks": 17
        }
    })


class SystemStatsResponse(BaseModel):
    """系统统计响应"""
    success: bool = Field(default=True, description="请求是否成功")
    data: SystemStatsData = Field(..., description="系统统计数据")
    namespace: str = Field(..., description="命名空间")
    timestamp: datetime = Field(default_factory=datetime.now, description="统计时间")

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "success": True,
            "data": {
                "total_queues": 5,
                "total_tasks": 1250,
                "active_workers": 12,
                "pending_tasks": 45,
                "processing_tasks": 8,
                "completed_tasks": 1180,
                "failed_tasks": 17
            },
            "namespace": "default",
            "timestamp": "2025-10-18T10:30:00Z"
        }
    })


class DashboardStatsData(BaseModel):
    """仪表板统计数据"""
    total_tasks: int = Field(..., description="任务总数", ge=0)
    success_count: int = Field(..., description="成功任务数", ge=0)
    failed_count: int = Field(..., description="失败任务数", ge=0)
    success_rate: float = Field(..., description="成功率 (0-100)", ge=0, le=100)
    throughput: float = Field(..., description="吞吐量 (任务/秒)", ge=0)
    avg_processing_time: float = Field(..., description="平均处理时间 (秒)", ge=0)

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "total_tasks": 1250,
            "success_count": 1180,
            "failed_count": 17,
            "success_rate": 94.4,
            "throughput": 12.5,
            "avg_processing_time": 2.3
        }
    })


class DashboardStatsResponse(BaseModel):
    """仪表板统计响应"""
    success: bool = Field(default=True, description="请求是否成功")
    data: DashboardStatsData = Field(..., description="仪表板统计数据")
    namespace: str = Field(..., description="命名空间")
    time_range: str = Field(..., description="时间范围")

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "success": True,
            "data": {
                "total_tasks": 1250,
                "success_count": 1180,
                "failed_count": 17,
                "success_rate": 94.4,
                "throughput": 12.5,
                "avg_processing_time": 2.3
            },
            "namespace": "default",
            "time_range": "24h"
        }
    })


class QueueRankingItem(BaseModel):
    """队列排行项"""
    queue_name: str = Field(..., description="队列名称")
    value: float = Field(..., description="指标值", ge=0)
    rank: int = Field(..., description="排名", ge=1)

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "queue_name": "email_queue",
            "value": 150.5,
            "rank": 1
        }
    })


class TopQueuesResponse(BaseModel):
    """队列排行榜响应"""
    success: bool = Field(default=True, description="请求是否成功")
    data: List[QueueRankingItem] = Field(..., description="排行榜数据")
    metric: str = Field(..., description="指标类型 (backlog/error)")
    namespace: str = Field(..., description="命名空间")
    time_range: str = Field(..., description="时间范围")

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "success": True,
            "data": [
                {"queue_name": "email_queue", "value": 150, "rank": 1},
                {"queue_name": "sms_queue", "value": 89, "rank": 2},
                {"queue_name": "push_queue", "value": 45, "rank": 3}
            ],
            "metric": "backlog",
            "namespace": "default",
            "time_range": "24h"
        }
    })



class NamespaceStatistics(BaseModel):
    """命名空间统计信息"""
    total_queues: int = Field(..., description="队列总数", ge=0)
    total_tasks: int = Field(..., description="任务总数", ge=0)
    active_workers: int = Field(..., description="活跃 Worker 数", ge=0)
    redis_memory_usage: Optional[int] = Field(None, description="Redis 内存使用 (bytes)", ge=0)
    db_connections: Optional[int] = Field(None, description="数据库连接数", ge=0)

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "total_queues": 5,
            "total_tasks": 1250,
            "active_workers": 12,
            "redis_memory_usage": 10485760,
            "db_connections": 5
        }
    })


class NamespaceStatisticsResponse(BaseModel):
    """命名空间统计响应"""
    success: bool = Field(default=True, description="请求是否成功")
    data: NamespaceStatistics = Field(..., description="统计数据")
    namespace: str = Field(..., description="命名空间名称")

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "success": True,
            "data": {
                "total_queues": 5,
                "total_tasks": 1250,
                "active_workers": 12,
                "redis_memory_usage": 10485760,
                "db_connections": 5
            },
            "namespace": "default"
        }
    })



class WorkerHeartbeatInfo(BaseModel):
    """Worker 心跳信息"""
    worker_id: str = Field(..., description="Worker ID")
    queue_name: str = Field(..., description="队列名称")
    status: str = Field(..., description="状态 (online/offline)")
    last_heartbeat: datetime = Field(..., description="最后心跳时间")
    tasks_processed: int = Field(..., description="已处理任务数", ge=0)
    current_task: Optional[str] = Field(None, description="当前处理的任务 ID")

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "worker_id": "worker-001",
            "queue_name": "email_queue",
            "status": "online",
            "last_heartbeat": "2025-10-18T10:30:00Z",
            "tasks_processed": 150,
            "current_task": "task-12345"
        }
    })


class WorkersResponse(BaseModel):
    """Workers 列表响应"""
    success: bool = Field(default=True, description="请求是否成功")
    namespace: str = Field(..., description="命名空间")
    queue: str = Field(..., description="队列名称")
    workers: List[WorkerHeartbeatInfo] = Field(..., description="Worker 列表")

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "success": True,
            "namespace": "default",
            "queue": "email_queue",
            "workers": [
                {
                    "worker_id": "worker-001",
                    "queue_name": "email_queue",
                    "status": "online",
                    "last_heartbeat": "2025-10-18T10:30:00Z",
                    "tasks_processed": 150,
                    "current_task": "task-12345"
                }
            ]
        }
    })


class WorkerSummary(BaseModel):
    """Worker 汇总统计"""
    total_workers: int = Field(..., description="总 Worker 数", ge=0)
    online_workers: int = Field(..., description="在线 Worker 数", ge=0)
    offline_workers: int = Field(..., description="离线 Worker 数", ge=0)
    total_tasks_processed: int = Field(..., description="总处理任务数", ge=0)
    avg_tasks_per_worker: float = Field(..., description="平均每个 Worker 处理任务数", ge=0)

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "total_workers": 12,
            "online_workers": 10,
            "offline_workers": 2,
            "total_tasks_processed": 1500,
            "avg_tasks_per_worker": 125.0
        }
    })


class WorkerSummaryResponse(BaseModel):
    """Worker 汇总统计响应"""
    success: bool = Field(default=True, description="请求是否成功")
    namespace: str = Field(..., description="命名空间")
    queue: str = Field(..., description="队列名称")
    summary: WorkerSummary = Field(..., description="汇总统计")

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "success": True,
            "namespace": "default",
            "queue": "email_queue",
            "summary": {
                "total_workers": 12,
                "online_workers": 10,
                "offline_workers": 2,
                "total_tasks_processed": 1500,
                "avg_tasks_per_worker": 125.0
            }
        }
    })


class WorkerOfflineRecord(BaseModel):
    """Worker 离线记录"""
    worker_id: str = Field(..., description="Worker ID")
    queue_name: str = Field(..., description="队列名称")
    offline_time: datetime = Field(..., description="离线时间")
    last_task: Optional[str] = Field(None, description="最后处理的任务 ID")
    reason: Optional[str] = Field(None, description="离线原因")

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "worker_id": "worker-002",
            "queue_name": "email_queue",
            "offline_time": "2025-10-18T09:30:00Z",
            "last_task": "task-12340",
            "reason": "heartbeat_timeout"
        }
    })


class WorkerOfflineHistoryResponse(BaseModel):
    """Worker 离线历史响应"""
    success: bool = Field(default=True, description="请求是否成功")
    history: List[WorkerOfflineRecord] = Field(..., description="离线历史记录")
    total: int = Field(..., description="总记录数", ge=0)

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "success": True,
            "history": [
                {
                    "worker_id": "worker-002",
                    "queue_name": "email_queue",
                    "offline_time": "2025-10-18T09:30:00Z",
                    "last_task": "task-12340",
                    "reason": "heartbeat_timeout"
                }
            ],
            "total": 1
        }
    })



class DatabaseStatus(BaseModel):
    """数据库状态"""
    connected: bool = Field(..., description="是否已连接")
    host: Optional[str] = Field(None, description="主机地址")
    port: Optional[int] = Field(None, description="端口")
    database: Optional[str] = Field(None, description="数据库名称")
    pool_size: Optional[int] = Field(None, description="连接池大小", ge=0)
    active_connections: Optional[int] = Field(None, description="活跃连接数", ge=0)

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "connected": True,
            "host": "localhost",
            "port": 5432,
            "database": "jettask",
            "pool_size": 10,
            "active_connections": 3
        }
    })


class SystemSettings(BaseModel):
    """系统设置"""
    api_version: str = Field(..., description="API 版本")
    service_name: str = Field(..., description="服务名称")
    environment: str = Field(..., description="运行环境 (development/staging/production)")
    debug_mode: bool = Field(..., description="是否开启调试模式")
    database: DatabaseStatus = Field(..., description="数据库状态")

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "api_version": "v1",
            "service_name": "JetTask WebUI",
            "environment": "development",
            "debug_mode": True,
            "database": {
                "connected": True,
                "host": "localhost",
                "port": 5432,
                "database": "jettask",
                "pool_size": 10,
                "active_connections": 3
            }
        }
    })


class SystemSettingsResponse(BaseModel):
    """系统设置响应"""
    success: bool = Field(default=True, description="请求是否成功")
    data: SystemSettings = Field(..., description="系统设置")

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "success": True,
            "data": {
                "api_version": "v1",
                "service_name": "JetTask WebUI",
                "environment": "development",
                "debug_mode": True,
                "database": {
                    "connected": True,
                    "host": "localhost",
                    "port": 5432,
                    "database": "jettask",
                    "pool_size": 10,
                    "active_connections": 3
                }
            }
        }
    })


class DatabaseStatusResponse(BaseModel):
    """数据库状态响应"""
    success: bool = Field(default=True, description="请求是否成功")
    data: DatabaseStatus = Field(..., description="数据库状态")

    model_config = ConfigDict(json_schema_extra={
        "example": {
            "success": True,
            "data": {
                "connected": True,
                "host": "localhost",
                "port": 5432,
                "database": "jettask",
                "pool_size": 10,
                "active_connections": 3
            }
        }
    })


__all__ = [
    'SuccessResponse',
    'DataResponse',
    'PaginatedDataResponse',

    'SystemStatsData',
    'SystemStatsResponse',
    'DashboardStatsData',
    'DashboardStatsResponse',
    'QueueRankingItem',
    'TopQueuesResponse',

    'NamespaceStatistics',
    'NamespaceStatisticsResponse',

    'WorkerHeartbeatInfo',
    'WorkersResponse',
    'WorkerSummary',
    'WorkerSummaryResponse',
    'WorkerOfflineRecord',
    'WorkerOfflineHistoryResponse',

    'DatabaseStatus',
    'SystemSettings',
    'SystemSettingsResponse',
    'DatabaseStatusResponse',
]



"""
SQL 历史查询模块 - SQL WHERE 条件的历史记录管理、模糊搜索和使用统计
"""
from fastapi import APIRouter, HTTPException, Request, Query, Path
from typing import Optional, Dict, Any

from jettask.webui.services.sql_history_service import SQLHistoryService
from jettask.utils.task_logger import get_task_logger

router = APIRouter(prefix="/sql-history", tags=["sql-history"])
logger = get_task_logger(__name__)


@router.get(
    "/search",
    summary="搜索 SQL 历史记录",
    description="""模糊搜索 SQL WHERE 查询历史记录，支持自动补全。

**主要功能**:
- 模糊匹配 WHERE 条件和别名
- 按使用次数 + 创建时间降序排列
- 支持按类别过滤（系统内置/用户历史）
- 使用 PostgreSQL pg_trgm 扩展实现高效搜索

**使用场景**: 前端输入框自动补全、查询建议""",
    responses={
        200: {
            "description": "成功返回匹配的 SQL 历史记录",
            "content": {
                "application/json": {
                    "example": {
                        "success": True,
                        "data": [
                            {
                                "id": 1,
                                "namespace": "default",
                                "where_clause": "status = 'success'",
                                "alias": "成功的任务",
                                "category": "system",
                                "usage_count": 150,
                                "created_at": "2025-11-18T00:00:00+00:00",
                                "last_used_at": "2025-11-18T10:30:00+00:00"
                            }
                        ]
                    }
                }
            }
        }
    }
)
async def search_sql_history(
    request: Request,
    keyword: Optional[str] = Query(None, description="搜索关键词（模糊匹配 WHERE 条件和别名）", example="status"),
    category: Optional[str] = Query(None, description="类别过滤: system（系统内置）/ user（用户历史），为空返回所有", example="user"),
    limit: int = Query(10, ge=1, le=50, description="返回数量限制", example=10)
) -> Dict[str, Any]:
    try:
        ns = request.state.ns
        pg_session = await ns.get_pg_session()

        results = await SQLHistoryService.search(
            pg_session=pg_session,
            namespace=ns.name,
            keyword=keyword,
            category=category,
            limit=limit
        )

        if category == 'system' and not results:
            logger.info(f"命名空间 '{ns.name}' 中没有系统内置查询，尝试从 'default' 复制")
            copied_count = await SQLHistoryService.copy_system_templates(
                pg_session=pg_session,
                target_namespace=ns.name,
                template_namespace='default'
            )

            if copied_count > 0:
                results = await SQLHistoryService.search(
                    pg_session=pg_session,
                    namespace=ns.name,
                    keyword=keyword,
                    category=category,
                    limit=limit
                )
                logger.info(f"复制成功，重新查询返回 {len(results)} 条记录")

        return {
            "success": True,
            "data": results
        }

    except Exception as e:
        logger.error(f"搜索 SQL 历史失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))


@router.delete(
    "/{history_id}",
    summary="删除 SQL 历史记录",
    description="""删除用户的历史记录（系统内置记录不可删除）。

**限制**: 只有 category='user' 的记录可以删除，系统内置（category='system'）不可删除""",
    responses={
        200: {
            "description": "成功删除记录",
            "content": {
                "application/json": {
                    "example": {
                        "success": True,
                        "message": "历史记录已删除"
                    }
                }
            }
        },
        404: {"description": "记录不存在或不可删除"}
    }
)
async def delete_sql_history(
    request: Request,
    history_id: int = Path(..., description="历史记录 ID", example=1)
) -> Dict[str, Any]:
    try:
        ns = request.state.ns
        pg_session = await ns.get_pg_session()

        success = await SQLHistoryService.delete_history(
            pg_session=pg_session,
            history_id=history_id
        )

        if success:
            return {
                "success": True,
                "message": "历史记录已删除"
            }
        else:
            raise HTTPException(status_code=404, detail="记录不存在或不可删除（系统内置记录不可删除）")

    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"删除 SQL 历史记录失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))



"""
队列服务层
处理队列相关的业务逻辑
"""
from typing import Optional, List, Dict, Any
from datetime import datetime, timedelta, timezone
from enum import Enum
from sqlalchemy import text, select, func, and_
from sqlalchemy.ext.asyncio import AsyncSession
import logging

from jettask.db.models.task import Task
from jettask.db.models.task_run import TaskRun
from jettask.db.models.task_metrics_minute import TaskMetricsMinute
from jettask.db.models.task_runs_metrics_minute import TaskRunsMetricsMinute
from jettask.webui.services.base_service import BaseService

logger = logging.getLogger(__name__)


class TrendMetric(str, Enum):
    """趋势查询支持的聚合指标（task_run 维度）"""
    TOTAL_COUNT = "total_count"          
    SUCCESS_COUNT = "success_count"       
    FAILED_COUNT = "failed_count"         
    RETRY_COUNT = "retry_count"           
    AVG_DURATION = "avg_duration"         
    MAX_DURATION = "max_duration"         
    MIN_DURATION = "min_duration"         
    AVG_DELAY = "avg_delay"               
    MAX_DELAY = "max_delay"               
    MIN_DELAY = "min_delay"               
    MAX_CONCURRENCY = "max_concurrency"   
    SUCCESS_RATE = "success_rate"         


class TaskTrendMetric(str, Enum):
    """趋势查询支持的聚合指标（task 维度）"""
    TASK_COUNT = "task_count"             


class QueueService(BaseService):
    """队列服务类"""
    
    @staticmethod
    def get_base_queue_name(queue_name: str) -> str:
        if ':' in queue_name:
            parts = queue_name.rsplit(':', 1)
            if parts[-1].isdigit():
                return parts[0]
        return queue_name
    
    @staticmethod
    async def get_queues_by_namespace(namespace_data_access, namespace: str) -> Dict[str, Any]:
        queues_data = await namespace_data_access.get_queue_stats(namespace)
        return {
            "success": True,
            "data": list(set([QueueService.get_base_queue_name(q['queue_name']) for q in queues_data]))
        }
    
    @staticmethod
    async def get_queue_flow_rates(data_access, query) -> Dict[str, Any]:
        now = datetime.now(timezone.utc)
        
        if query.start_time and query.end_time:
            start_time = query.start_time
            end_time = query.end_time
            logger.info(f"使用自定义时间范围: {start_time} 到 {end_time}")
        else:
            time_range_map = {
                "15m": timedelta(minutes=15),
                "30m": timedelta(minutes=30),
                "1h": timedelta(hours=1),
                "3h": timedelta(hours=3),
                "6h": timedelta(hours=6),
                "12h": timedelta(hours=12),
                "24h": timedelta(hours=24),
                "7d": timedelta(days=7),
                "30d": timedelta(days=30),
            }
            
            time_range_value = query.time_range if query.time_range else query.interval
            delta = time_range_map.get(time_range_value, timedelta(minutes=15))
            
            queue_name = query.queues[0] if query.queues else None
            if queue_name:
                latest_time = await data_access.get_latest_task_time(queue_name)
                if latest_time:
                    end_time = latest_time.replace(second=59, microsecond=999999)  
                    logger.info(f"使用最新任务时间: {latest_time}")
                else:
                    end_time = now.replace(second=0, microsecond=0)
            else:
                end_time = now.replace(second=0, microsecond=0)
            
            start_time = end_time - delta
            logger.info(f"使用预设时间范围 {time_range_value}: {start_time} 到 {end_time}, delta: {delta}")
        
        if not query.queues or len(query.queues) == 0:
            return {"data": [], "granularity": "minute"}
        
        queue_name = query.queues[0]
        filters = getattr(query, 'filters', None)
        data, granularity = await data_access.fetch_queue_flow_rates(
            queue_name, start_time, end_time, filters
        )
        
        return {"data": data, "granularity": granularity}
    
    @staticmethod
    async def get_global_stats(data_access) -> Dict[str, Any]:
        stats_data = await data_access.fetch_global_stats()
        return {
            "success": True,
            "data": stats_data
        }
    
    @staticmethod
    async def get_queues_detail(data_access) -> Dict[str, Any]:
        queues_data = await data_access.fetch_queues_data()
        return {
            "success": True,
            "data": queues_data
        }
    
    @staticmethod
    async def delete_queue(queue_name: str) -> Dict[str, Any]:
        logger.info(f"删除队列请求: {queue_name}")
        return {
            "success": True,
            "message": f"队列 {queue_name} 已删除"
        }
    
    @staticmethod
    async def trim_queue(queue_name: str, max_length: int) -> Dict[str, Any]:
        logger.info(f"裁剪队列请求: {queue_name}, 保留 {max_length} 条消息")
        return {
            "success": True,
            "message": f"队列 {queue_name} 已裁剪至 {max_length} 条消息"
        }
    
    @staticmethod
    async def get_queue_stats_v2(
        namespace_data_access,
        namespace: str,
        queue: Optional[str] = None,
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        time_range: Optional[str] = None
    ) -> Dict[str, Any]:
        conn = await namespace_data_access.manager.get_connection(namespace)
        
        redis_client = await conn.get_redis_client(decode=False)
        
        pg_session = None
        if conn.AsyncSessionLocal:
            pg_session = conn.AsyncSessionLocal()
        
        try:
            from jettask.webui.services.queue_stats_v2 import QueueStatsV2
            
            stats_service = QueueStatsV2(
                redis_client=redis_client,
                pg_session=pg_session,
                redis_prefix=conn.redis_prefix
            )
            
            time_filter = None
            if time_range or start_time or end_time:
                time_filter = {}
                
                if time_range and time_range != 'custom':
                    now = datetime.now(timezone.utc)
                    if time_range.endswith('m'):
                        minutes = int(time_range[:-1])
                        time_filter['start_time'] = now - timedelta(minutes=minutes)
                        time_filter['end_time'] = now
                    elif time_range.endswith('h'):
                        hours = int(time_range[:-1])
                        time_filter['start_time'] = now - timedelta(hours=hours)
                        time_filter['end_time'] = now
                    elif time_range.endswith('d'):
                        days = int(time_range[:-1])
                        time_filter['start_time'] = now - timedelta(days=days)
                        time_filter['end_time'] = now
                else:
                    if start_time:
                        time_filter['start_time'] = start_time
                    if end_time:
                        time_filter['end_time'] = end_time
            
            stats = await stats_service.get_queue_stats_grouped(time_filter)
            
            if queue:
                stats = [s for s in stats if s['queue_name'] == queue]
            
            return {
                "success": True,
                "data": stats
            }
            
        finally:
            if pg_session:
                await pg_session.close()
            await redis_client.aclose()
    
    @staticmethod
    async def get_tasks_v2(namespace_data_access, namespace: str, body: Dict[str, Any]) -> Dict[str, Any]:
        from sqlalchemy import text
        from datetime import datetime, timezone, timedelta
        
        queue_name = body.get('queue_name')
        page = body.get('page', 1)
        page_size = body.get('page_size', 20)
        filters = body.get('filters', [])
        time_range = body.get('time_range', '1h')
        start_time = body.get('start_time')
        end_time = body.get('end_time')
        sort_field = body.get('sort_field', 'created_at')
        sort_order = body.get('sort_order', 'desc')
        
        if not queue_name:
            raise ValueError("queue_name is required")
        
        conn = await namespace_data_access.manager.get_connection(namespace)
        
        if not conn.pg_config or not conn.async_engine:
            return {
                "success": True,
                "data": [],
                "total": 0
            }
        
        if start_time and end_time:
            start_dt = datetime.fromisoformat(start_time.replace('Z', '+00:00'))
            end_dt = datetime.fromisoformat(end_time.replace('Z', '+00:00'))
        else:
            end_dt = datetime.now(timezone.utc)
            time_deltas = {
                '15m': timedelta(minutes=15),
                '30m': timedelta(minutes=30),
                '1h': timedelta(hours=1),
                '3h': timedelta(hours=3),
                '6h': timedelta(hours=6),
                '12h': timedelta(hours=12),
                '1d': timedelta(days=1),
                '3d': timedelta(days=3),
                '7d': timedelta(days=7),
                '30d': timedelta(days=30)
            }
            delta = time_deltas.get(time_range, timedelta(hours=1))
            start_dt = end_dt - delta
        
        offset = (page - 1) * page_size
        
        async with conn.async_engine.begin() as pg_conn:
            conditions = [
                "t.namespace = :namespace",
                "t.queue = :queue",
                "t.created_at >= :start_time",
                "t.created_at <= :end_time"
            ]
            query_params = {
                "namespace": namespace,
                "queue": queue_name,
                "start_time": start_dt,
                "end_time": end_dt,
                "limit": page_size,
                "offset": offset
            }
            
            for i, filter_item in enumerate(filters):
                field = filter_item.get('field')
                operator = filter_item.get('operator')
                value = filter_item.get('value')
                
                if field and operator and value is not None:
                    param_key = f"filter_{i}"
                    
                    db_field_map = {
                        'id': 't.stream_id',
                        'task_name': "t.payload::jsonb->'event_data'->>'__task_name'",
                        'status': "t.payload::jsonb->>'status'",
                        'worker_id': "t.payload::jsonb->>'worker_id'",
                        'scheduled_task_id': 't.scheduled_task_id'
                    }
                    
                    db_field = db_field_map.get(field, f't.{field}')
                    
                    if operator == 'eq':
                        conditions.append(f"{db_field} = :{param_key}")
                        query_params[param_key] = value
                    elif operator == 'contains':
                        conditions.append(f"{db_field} LIKE :{param_key}")
                        query_params[param_key] = f"%{value}%"
                    elif operator == 'gt':
                        conditions.append(f"{db_field} > :{param_key}")
                        query_params[param_key] = value
                    elif operator == 'lt':
                        conditions.append(f"{db_field} < :{param_key}")
                        query_params[param_key] = value
            
            where_clause = " AND ".join(conditions)
            
            count_query = f"""
                SELECT COUNT(*) as total 
                FROM tasks t
                WHERE {where_clause}
            """
            count_result = await pg_conn.execute(text(count_query), query_params)
            total = count_result.fetchone().total
            
            sort_map = {
                'created_at': 't.created_at',
                'started_at': 't.started_at',
                'completed_at': 't.completed_at'
            }
            order_by = sort_map.get(sort_field, 't.created_at')
            order_direction = 'DESC' if sort_order == 'desc' else 'ASC'
            
            query = f"""
                SELECT 
                    t.stream_id as id,
                    t.payload::jsonb->'event_data'->>'__task_name' as task_name,
                    t.queue,
                    t.payload::jsonb->>'status' as status,
                    t.priority,
                    COALESCE((t.payload::jsonb->>'retry_count')::int, 0) as retry_count,
                    COALESCE((t.payload::jsonb->>'max_retry')::int, 3) as max_retry,
                    t.created_at,
                    (t.payload::jsonb->>'started_at')::timestamptz as started_at,
                    (t.payload::jsonb->>'completed_at')::timestamptz as completed_at,
                    t.payload::jsonb->>'worker_id' as worker_id,
                    t.payload::jsonb->>'error_message' as error_message,
                    (t.payload::jsonb->>'execution_time')::float as execution_time,
                    CASE 
                        WHEN t.payload::jsonb->>'completed_at' IS NOT NULL AND t.created_at IS NOT NULL 
                        THEN EXTRACT(EPOCH FROM ((t.payload::jsonb->>'completed_at')::timestamptz - t.created_at))
                        ELSE NULL 
                    END as duration,
                    t.scheduled_task_id,
                    t.source,
                    t.metadata
                FROM tasks t
                WHERE {where_clause}
                ORDER BY {order_by} {order_direction}
                LIMIT :limit OFFSET :offset
            """
            
            result = await pg_conn.execute(text(query), query_params)
            
            tasks = []
            for row in result:
                tasks.append({
                    "id": row.id,
                    "task_name": row.task_name or "unknown",
                    "queue": row.queue,
                    "status": row.status,
                    "priority": row.priority,
                    "retry_count": row.retry_count,
                    "max_retry": row.max_retry,
                    "created_at": row.created_at.isoformat() if row.created_at else None,
                    "started_at": row.started_at.isoformat() if row.started_at else None,
                    "completed_at": row.completed_at.isoformat() if row.completed_at else None,
                    "duration": round(row.duration, 2) if row.duration else None,
                    "execution_time": float(row.execution_time) if row.execution_time else None,
                    "worker_id": row.worker_id,
                    "error_message": row.error_message
                })
            
            return {
                "success": True,
                "data": tasks,
                "total": total
            }
    
    @staticmethod
    async def get_consumer_group_stats(namespace_data_access, namespace: str, group_name: str) -> Dict[str, Any]:
        conn = await namespace_data_access.manager.get_connection(namespace)
        
        if not conn.AsyncSessionLocal:
            raise ValueError("PostgreSQL not configured for this namespace")
        
        async with conn.AsyncSessionLocal() as session:
            query = text("""
                WITH group_stats AS (
                    SELECT 
                        tr.consumer_group,
                        tr.task_name,
                        COUNT(*) as total_tasks,
                        COUNT(CASE WHEN tr.status = 'success' THEN 1 END) as success_count,
                        COUNT(CASE WHEN tr.status = 'failed' THEN 1 END) as failed_count,
                        COUNT(CASE WHEN tr.status = 'running' THEN 1 END) as running_count,
                        AVG(tr.execution_time) as avg_execution_time,
                        MIN(tr.execution_time) as min_execution_time,
                        MAX(tr.execution_time) as max_execution_time,
                        AVG(tr.duration) as avg_duration,
                        MIN(tr.started_at) as first_task_time,
                        MAX(tr.completed_at) as last_task_time
                    FROM task_runs tr
                    WHERE tr.consumer_group = :group_name
                        AND tr.started_at > NOW() - INTERVAL '24 hours'
                    GROUP BY tr.consumer_group, tr.task_name
                ),
                hourly_stats AS (
                    SELECT 
                        DATE_TRUNC('hour', tr.started_at) as hour,
                        COUNT(*) as task_count,
                        AVG(tr.execution_time) as avg_exec_time
                    FROM task_runs tr
                    WHERE tr.consumer_group = :group_name
                        AND tr.started_at > NOW() - INTERVAL '24 hours'
                    GROUP BY DATE_TRUNC('hour', tr.started_at)
                    ORDER BY hour
                )
                SELECT 
                    (SELECT row_to_json(gs) FROM group_stats gs) as summary,
                    (SELECT json_agg(hs) FROM hourly_stats hs) as hourly_trend
            """)
            
            result = await session.execute(query, {'group_name': group_name})
            row = result.fetchone()
            
            if not row or not row.summary:
                return {
                    "success": True,
                    "data": {
                        "group_name": group_name,
                        "summary": {},
                        "hourly_trend": []
                    }
                }
            
            return {
                "success": True,
                "data": {
                    "group_name": group_name,
                    "summary": row.summary,
                    "hourly_trend": row.hourly_trend or []
                }
            }
    
    @staticmethod
    async def get_stream_backlog(
        data_access,
        namespace: str,
        stream_name: Optional[str] = None,
        hours: int = 24
    ) -> Dict[str, Any]:
        return {
            'success': False,
            'data': [],
            'total': 0,
            'message': 'stream_backlog_monitor 表已废弃，请使用其他积压监控接口'
        }
    
    @staticmethod
    async def get_stream_backlog_summary(data_access, namespace: str) -> Dict[str, Any]:
        return {
            'success': False,
            'data': {
                'total_streams': 0,
                'total_groups': 0,
                'total_backlog': 0,
                'total_pending': 0,
                'max_backlog': 0
            },
            'message': 'stream_backlog_monitor 表已废弃，请使用其他积压监控接口'
        }


    @classmethod
    async def get_queue_overview(
        cls,
        namespace: str,
        pg_session: AsyncSession,
        registry,  
        queues: List[str],  
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        time_range: Optional[str] = None
    ) -> Dict[str, Any]:
        try:
            time_range_result = cls._resolve_time_range(
                start_time=start_time,
                end_time=end_time,
                time_range=time_range,
                default_range="1h",
                min_interval_seconds=60  
            )
            start_time = time_range_result.start_time
            end_time = time_range_result.end_time

            queue_result = await cls._resolve_queues(queues, registry, expand_priority=True)
            target_queues = queue_result.target_queues
            all_priority_queues = queue_result.all_priority_queues
            base_queue_map = queue_result.base_queue_map

            if not target_queues:
                logger.warning(f"请求的队列 {queues} 在命名空间 {namespace} 中不存在")
                return {
                    "success": True,
                    "data": [],
                    "total": 0,
                    "granularity": time_range_result.granularity,
                    "time_range": {
                        "start_time": start_time.isoformat() if start_time else None,
                        "end_time": end_time.isoformat() if end_time else None,
                        "interval": time_range_result.interval,
                        "interval_seconds": time_range_result.interval_seconds
                    }
                }

            logger.info(
                f"队列概览查询: namespace={namespace}, queues={target_queues}, "
                f"priority_queues={all_priority_queues}, "
                f"time_range={start_time} to {end_time}, "
                f"granularity={time_range_result.granularity}"
            )

            stats_stmt = select(
                TaskRunsMetricsMinute.queue,
                TaskRunsMetricsMinute.task_name,
                func.sum(TaskRunsMetricsMinute.total_count).label('total_count'),
                func.sum(TaskRunsMetricsMinute.success_count).label('success_count'),
                func.sum(TaskRunsMetricsMinute.failed_count).label('failed_count'),
                func.sum(TaskRunsMetricsMinute.retry_count).label('retry_count'),
                func.sum(TaskRunsMetricsMinute.total_duration).label('total_duration'),
                func.max(TaskRunsMetricsMinute.max_duration).label('max_duration'),
                func.min(TaskRunsMetricsMinute.min_duration).label('min_duration'),
                func.sum(TaskRunsMetricsMinute.total_delay).label('total_delay'),
                func.max(TaskRunsMetricsMinute.max_delay).label('max_delay'),
                func.min(TaskRunsMetricsMinute.min_delay).label('min_delay'),
                func.max(TaskRunsMetricsMinute.running_concurrency).label('max_concurrency')
            ).where(
                and_(
                    TaskRunsMetricsMinute.queue.in_(all_priority_queues),
                    TaskRunsMetricsMinute.namespace == namespace,
                    TaskRunsMetricsMinute.time_bucket >= start_time,
                    TaskRunsMetricsMinute.time_bucket <= end_time
                )
            ).group_by(TaskRunsMetricsMinute.queue, TaskRunsMetricsMinute.task_name)

            result = await pg_session.execute(stats_stmt)
            rows = result.all()

            base_queue_tasks = {}
            for base_queue in target_queues:
                base_queue_tasks[base_queue] = {}

            for row in rows:
                priority_queue = row.queue
                base_queue = base_queue_map.get(priority_queue, cls.get_base_queue_name(priority_queue))
                task_name = row.task_name

                if base_queue not in base_queue_tasks:
                    continue

                if task_name not in base_queue_tasks[base_queue]:
                    base_queue_tasks[base_queue][task_name] = {
                        "task_name": task_name,
                        "total_count": 0,
                        "success_count": 0,
                        "failed_count": 0,
                        "retry_count": 0,
                        "total_duration": 0.0,
                        "max_duration": None,
                        "min_duration": None,
                        "total_delay": 0.0,
                        "max_delay": None,
                        "min_delay": None,
                        "max_concurrency": 0
                    }

                stats = base_queue_tasks[base_queue][task_name]
                stats["total_count"] += row.total_count or 0
                stats["success_count"] += row.success_count or 0
                stats["failed_count"] += row.failed_count or 0
                stats["retry_count"] += row.retry_count or 0
                stats["total_duration"] += row.total_duration or 0.0
                stats["total_delay"] += row.total_delay or 0.0
                if row.max_duration is not None:
                    stats["max_duration"] = max(stats["max_duration"] or 0, row.max_duration)
                if row.max_delay is not None:
                    stats["max_delay"] = max(stats["max_delay"] or 0, row.max_delay)
                if row.max_concurrency is not None:
                    stats["max_concurrency"] = max(stats["max_concurrency"], row.max_concurrency)
                if row.min_duration is not None:
                    if stats["min_duration"] is None:
                        stats["min_duration"] = row.min_duration
                    else:
                        stats["min_duration"] = min(stats["min_duration"], row.min_duration)
                if row.min_delay is not None:
                    if stats["min_delay"] is None:
                        stats["min_delay"] = row.min_delay
                    else:
                        stats["min_delay"] = min(stats["min_delay"], row.min_delay)

            overview_data = []
            for queue_name in sorted(target_queues):
                tasks_data = []
                for task_name, stats in base_queue_tasks[queue_name].items():
                    if stats["total_count"] > 0:
                        stats["avg_duration"] = round(stats["total_duration"] / stats["total_count"], 4)
                        stats["avg_delay"] = round(stats["total_delay"] / stats["total_count"], 4)
                        stats["success_rate"] = round(
                            (stats["success_count"] / (stats["success_count"] + stats["failed_count"])) * 100, 2
                        ) if (stats["success_count"] + stats["failed_count"]) > 0 else 0.0
                    else:
                        stats["avg_duration"] = 0.0
                        stats["avg_delay"] = 0.0
                        stats["success_rate"] = 0.0

                    stats["total_duration"] = round(stats["total_duration"], 4)
                    stats["total_delay"] = round(stats["total_delay"], 4)
                    if stats["max_duration"] is not None:
                        stats["max_duration"] = round(stats["max_duration"], 4)
                    if stats["min_duration"] is not None:
                        stats["min_duration"] = round(stats["min_duration"], 4)
                    if stats["max_delay"] is not None:
                        stats["max_delay"] = round(stats["max_delay"], 4)
                    if stats["min_delay"] is not None:
                        stats["min_delay"] = round(stats["min_delay"], 4)

                    tasks_data.append(stats)

                tasks_data.sort(key=lambda x: x["task_name"])

                overview_data.append({
                    "queue_name": queue_name,
                    "task_count": len(tasks_data),
                    "tasks": tasks_data
                })

            return {
                "success": True,
                "data": overview_data,
                "total": len(overview_data),
                "granularity": time_range_result.granularity,
                "time_range": {
                    "start_time": start_time.isoformat() if start_time else None,
                    "end_time": end_time.isoformat() if end_time else None,
                    "interval": time_range_result.interval,
                    "interval_seconds": time_range_result.interval_seconds
                }
            }

        except Exception as e:
            logger.error(f"获取队列概览失败: {e}", exc_info=True)
            return {
                "success": False,
                "error": str(e),
                "data": [],
                "granularity": "minute",
                "time_range": {}
            }

    @classmethod
    async def get_queue_tasks_detail(
        cls,
        namespace: str,
        queue_name: str,
        pg_session: AsyncSession,
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        time_range: Optional[str] = None
    ) -> Dict[str, Any]:
        try:
            time_range_result = cls._resolve_time_range(
                start_time=start_time,
                end_time=end_time,
                time_range=time_range,
                default_range="15m"
            )
            start_time = time_range_result.start_time
            end_time = time_range_result.end_time

            logger.info(f"获取队列 tasks 详情: namespace={namespace}, queue={queue_name}, time_range={start_time} to {end_time}")

            task_stats_stmt = select(
                TaskRun.task_name,
                func.count().filter(TaskRun.status == 'success').label('success_count'),
                func.count().filter(TaskRun.status == 'failed').label('failed_count'),
                func.avg(TaskRun.duration).label('avg_duration')
            ).select_from(TaskRun).join(
                Task, TaskRun.stream_id == Task.stream_id
            ).where(
                and_(
                    Task.queue == queue_name,
                    Task.namespace == namespace,
                    TaskRun.created_at >= start_time,
                    TaskRun.created_at <= end_time,
                    TaskRun.task_name.isnot(None)
                )
            ).group_by(TaskRun.task_name)

            result = await pg_session.execute(task_stats_stmt)
            rows = result.all()

            tasks = []
            for row in rows:
                total_count = row.success_count + row.failed_count
                success_rate = round((row.success_count / total_count) * 100, 2) if total_count > 0 else 0.0

                tasks.append({
                    "task_name": row.task_name,
                    "success_count": row.success_count or 0,
                    "failed_count": row.failed_count or 0,
                    "success_rate": success_rate,
                    "avg_duration": round(row.avg_duration, 4) if row.avg_duration else 0.0
                })

            return {
                "success": True,
                "queue_name": queue_name,
                "tasks": tasks,
                "total": len(tasks),
                "time_range": {
                    "start_time": start_time.isoformat() if start_time else None,
                    "end_time": end_time.isoformat() if end_time else None
                }
            }

        except Exception as e:
            logger.error(f"获取队列 tasks 详情失败: queue={queue_name}, error={e}", exc_info=True)
            return {
                "success": False,
                "error": str(e),
                "queue_name": queue_name,
                "tasks": []
            }

    @classmethod
    async def get_queue_runs_trend(
        cls,
        namespace: str,
        pg_session: AsyncSession,
        registry,  
        queues: List[str],
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        time_range: Optional[str] = None,
        metrics: Optional[List[TrendMetric]] = None
    ) -> Dict[str, Any]:
        try:
            if metrics is None:
                metrics = [TrendMetric.TOTAL_COUNT]

            time_range_result = cls._resolve_time_range(
                start_time=start_time,
                end_time=end_time,
                time_range=time_range,
                default_range="1h",
                min_interval_seconds=60  
            )
            start_time = time_range_result.start_time
            end_time = time_range_result.end_time

            queue_result = await cls._resolve_queues(queues, registry, expand_priority=True)
            target_queues = queue_result.target_queues
            all_priority_queues = queue_result.all_priority_queues
            base_queue_map = queue_result.base_queue_map

            logger.info(
                f"队列趋势查询: namespace={namespace}, queues={target_queues}, "
                f"priority_queues={all_priority_queues}, metrics={[m.value for m in metrics]}, "
                f"time_range={start_time} to {end_time}, "
                f"interval={time_range_result.interval}, granularity={time_range_result.granularity}"
            )

            if not target_queues:
                return {
                    "success": True,
                    "data": [],
                    "metrics": [m.value for m in metrics],
                    "granularity": time_range_result.granularity,
                    "time_range": {
                        "start_time": start_time.isoformat(),
                        "end_time": end_time.isoformat(),
                        "interval": time_range_result.interval,
                        "interval_seconds": time_range_result.interval_seconds
                    }
                }

            interval_seconds = time_range_result.interval_seconds

            data = await cls._query_trend_from_runs_metrics(
                pg_session,
                namespace,
                target_queues,
                all_priority_queues,
                base_queue_map,
                start_time,
                end_time,
                interval_seconds,
                metrics
            )

            return {
                "success": True,
                "data": data,
                "metrics": [m.value for m in metrics],
                "granularity": time_range_result.granularity,
                "time_range": {
                    "start_time": start_time.isoformat(),
                    "end_time": end_time.isoformat(),
                    "interval": time_range_result.interval,
                    "interval_seconds": time_range_result.interval_seconds
                }
            }

        except Exception as e:
            logger.error(f"获取队列趋势失败: queues={queues}, error={e}", exc_info=True)
            return {
                "success": False,
                "error": str(e),
                "data": [],
                "metrics": [m.value for m in (metrics or [TrendMetric.TOTAL_COUNT])]
            }

    @classmethod
    async def get_queue_tasks_trend(
        cls,
        namespace: str,
        pg_session: AsyncSession,
        registry,  
        queues: List[str],
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        time_range: Optional[str] = None
    ) -> Dict[str, Any]:
        try:
            time_range_result = cls._resolve_time_range(
                start_time=start_time,
                end_time=end_time,
                time_range=time_range,
                default_range="1h",
                min_interval_seconds=60  
            )
            start_time = time_range_result.start_time
            end_time = time_range_result.end_time

            queue_result = await cls._resolve_queues(queues, registry, expand_priority=True)
            target_queues = queue_result.target_queues
            all_priority_queues = queue_result.all_priority_queues
            base_queue_map = queue_result.base_queue_map

            logger.info(
                f"队列任务创建趋势查询: namespace={namespace}, queues={target_queues}, "
                f"priority_queues={all_priority_queues}, "
                f"time_range={start_time} to {end_time}, "
                f"interval={time_range_result.interval}, granularity={time_range_result.granularity}"
            )

            if not target_queues:
                return {
                    "success": True,
                    "data": [],
                    "metrics": ["task_count"],
                    "granularity": time_range_result.granularity,
                    "time_range": {
                        "start_time": start_time.isoformat(),
                        "end_time": end_time.isoformat(),
                        "interval": time_range_result.interval,
                        "interval_seconds": time_range_result.interval_seconds
                    }
                }

            interval_seconds = time_range_result.interval_seconds

            stmt = select(
                TaskMetricsMinute.queue,
                TaskMetricsMinute.time_bucket,
                TaskMetricsMinute.task_count
            ).where(
                and_(
                    TaskMetricsMinute.namespace == namespace,
                    TaskMetricsMinute.queue.in_(all_priority_queues),
                    TaskMetricsMinute.time_bucket >= start_time,
                    TaskMetricsMinute.time_bucket <= end_time
                )
            ).order_by(
                TaskMetricsMinute.queue,
                TaskMetricsMinute.time_bucket
            )

            result = await pg_session.execute(stmt)
            rows = result.fetchall()

            time_buckets = []
            if interval_seconds > 60:
                start_timestamp = int(start_time.timestamp() / interval_seconds) * interval_seconds
                current = datetime.fromtimestamp(start_timestamp, tz=timezone.utc)
                end_timestamp = int(end_time.timestamp() / interval_seconds) * interval_seconds
                end_aligned = datetime.fromtimestamp(end_timestamp, tz=timezone.utc)
            else:
                current = start_time.replace(second=0, microsecond=0)
                end_aligned = end_time.replace(second=0, microsecond=0)

            while current <= end_aligned:
                time_buckets.append(current)
                current += timedelta(seconds=interval_seconds)

            queue_data_map = {}
            for base_queue in target_queues:
                queue_data_map[base_queue] = {bucket: 0 for bucket in time_buckets}

            for row in rows:
                priority_queue = row.queue
                base_queue = base_queue_map.get(priority_queue)
                if not base_queue or base_queue not in queue_data_map:
                    continue

                time_bucket = row.time_bucket
                count = row.task_count or 0

                if interval_seconds > 60:
                    bucket_timestamp = int(time_bucket.timestamp() / interval_seconds) * interval_seconds
                    aligned_bucket = datetime.fromtimestamp(bucket_timestamp, tz=timezone.utc)
                else:
                    aligned_bucket = time_bucket

                if aligned_bucket in queue_data_map[base_queue]:
                    queue_data_map[base_queue][aligned_bucket] += count

            data = []
            for base_queue in target_queues:
                trend = []
                for bucket in sorted(time_buckets):
                    trend.append({
                        "time": bucket.isoformat(),
                        "task_count": queue_data_map[base_queue][bucket]
                    })

                data.append({
                    "queue_name": base_queue,
                    "trend": trend
                })

            return {
                "success": True,
                "data": data,
                "metrics": ["task_count"],
                "granularity": time_range_result.granularity,
                "time_range": {
                    "start_time": start_time.isoformat(),
                    "end_time": end_time.isoformat(),
                    "interval": time_range_result.interval,
                    "interval_seconds": time_range_result.interval_seconds
                }
            }

        except Exception as e:
            logger.error(f"获取队列任务创建趋势失败: queues={queues}, error={e}", exc_info=True)
            return {
                "success": False,
                "error": str(e),
                "data": [],
                "metrics": ["task_count"]
            }

    @classmethod
    async def _query_trend_from_runs_metrics(
        cls,
        pg_session: AsyncSession,
        namespace: str,
        target_queues: List[str],
        all_priority_queues: List[str],
        base_queue_map: Dict[str, str],
        start_time: datetime,
        end_time: datetime,
        interval_seconds: int,
        metrics: List[TrendMetric]
    ) -> List[Dict[str, Any]]:
        select_fields = [
            TaskRunsMetricsMinute.queue,
            TaskRunsMetricsMinute.time_bucket,
        ]

        metric_fields = {}
        for metric in metrics:
            if metric == TrendMetric.TOTAL_COUNT:
                metric_fields['total_count'] = TaskRunsMetricsMinute.total_count
            elif metric == TrendMetric.SUCCESS_COUNT:
                metric_fields['success_count'] = TaskRunsMetricsMinute.success_count
            elif metric == TrendMetric.FAILED_COUNT:
                metric_fields['failed_count'] = TaskRunsMetricsMinute.failed_count
            elif metric == TrendMetric.RETRY_COUNT:
                metric_fields['retry_count'] = TaskRunsMetricsMinute.retry_count
            elif metric in [TrendMetric.AVG_DURATION, TrendMetric.MAX_DURATION, TrendMetric.MIN_DURATION]:
                metric_fields['total_duration'] = TaskRunsMetricsMinute.total_duration
                metric_fields['max_duration'] = TaskRunsMetricsMinute.max_duration
                metric_fields['min_duration'] = TaskRunsMetricsMinute.min_duration
                if 'total_count' not in metric_fields:
                    metric_fields['total_count'] = TaskRunsMetricsMinute.total_count
            elif metric in [TrendMetric.AVG_DELAY, TrendMetric.MAX_DELAY, TrendMetric.MIN_DELAY]:
                metric_fields['total_delay'] = TaskRunsMetricsMinute.total_delay
                metric_fields['max_delay'] = TaskRunsMetricsMinute.max_delay
                metric_fields['min_delay'] = TaskRunsMetricsMinute.min_delay
                if 'total_count' not in metric_fields:
                    metric_fields['total_count'] = TaskRunsMetricsMinute.total_count
            elif metric == TrendMetric.MAX_CONCURRENCY:
                metric_fields['running_concurrency'] = TaskRunsMetricsMinute.running_concurrency
            elif metric == TrendMetric.SUCCESS_RATE:
                if 'success_count' not in metric_fields:
                    metric_fields['success_count'] = TaskRunsMetricsMinute.success_count
                if 'failed_count' not in metric_fields:
                    metric_fields['failed_count'] = TaskRunsMetricsMinute.failed_count

        for field in metric_fields.values():
            select_fields.append(field)

        stmt = select(*select_fields).where(
            and_(
                TaskRunsMetricsMinute.namespace == namespace,
                TaskRunsMetricsMinute.queue.in_(all_priority_queues),
                TaskRunsMetricsMinute.time_bucket >= start_time,
                TaskRunsMetricsMinute.time_bucket <= end_time
            )
        ).order_by(
            TaskRunsMetricsMinute.queue,
            TaskRunsMetricsMinute.time_bucket
        )

        result = await pg_session.execute(stmt)
        rows = result.fetchall()

        time_buckets = []
        if interval_seconds > 60:
            start_timestamp = int(start_time.timestamp() / interval_seconds) * interval_seconds
            current = datetime.fromtimestamp(start_timestamp, tz=timezone.utc)
            end_timestamp = int(end_time.timestamp() / interval_seconds) * interval_seconds
            end_aligned = datetime.fromtimestamp(end_timestamp, tz=timezone.utc)
        else:
            current = start_time.replace(second=0, microsecond=0)
            end_aligned = end_time.replace(second=0, microsecond=0)

        while current <= end_aligned:
            time_buckets.append(current)
            current += timedelta(seconds=interval_seconds)

        queue_data_map = {}
        for base_queue in target_queues:
            queue_data_map[base_queue] = {}
            for bucket in time_buckets:
                queue_data_map[base_queue][bucket] = cls._init_metric_values(metrics)

        for row in rows:
            priority_queue = row.queue
            base_queue = base_queue_map.get(priority_queue)
            if not base_queue or base_queue not in queue_data_map:
                continue

            time_bucket = row.time_bucket

            if interval_seconds > 60:
                bucket_timestamp = int(time_bucket.timestamp() / interval_seconds) * interval_seconds
                aligned_bucket = datetime.fromtimestamp(bucket_timestamp, tz=timezone.utc)
            else:
                aligned_bucket = time_bucket

            if aligned_bucket not in queue_data_map[base_queue]:
                continue

            bucket_data = queue_data_map[base_queue][aligned_bucket]
            cls._aggregate_metric_row(bucket_data, row, metric_fields, metrics)

        data = []
        for base_queue in target_queues:
            trend = []
            for bucket in sorted(time_buckets):
                bucket_data = queue_data_map[base_queue][bucket]
                point = {"time": bucket.isoformat()}

                for metric in metrics:
                    if metric == TrendMetric.TOTAL_COUNT:
                        point["total_count"] = bucket_data.get("_total_count", 0)
                    elif metric == TrendMetric.SUCCESS_COUNT:
                        point["success_count"] = bucket_data.get("_success_count", 0)
                    elif metric == TrendMetric.FAILED_COUNT:
                        point["failed_count"] = bucket_data.get("_failed_count", 0)
                    elif metric == TrendMetric.RETRY_COUNT:
                        point["retry_count"] = bucket_data.get("_retry_count", 0)
                    elif metric == TrendMetric.AVG_DURATION:
                        total_count = bucket_data.get("_total_count", 0)
                        total_duration = bucket_data.get("_total_duration", 0)
                        point["avg_duration"] = round(total_duration / total_count, 4) if total_count > 0 else 0
                    elif metric == TrendMetric.MAX_DURATION:
                        point["max_duration"] = bucket_data.get("_max_duration")
                    elif metric == TrendMetric.MIN_DURATION:
                        point["min_duration"] = bucket_data.get("_min_duration")
                    elif metric == TrendMetric.AVG_DELAY:
                        total_count = bucket_data.get("_total_count", 0)
                        total_delay = bucket_data.get("_total_delay", 0)
                        point["avg_delay"] = round(total_delay / total_count, 4) if total_count > 0 else 0
                    elif metric == TrendMetric.MAX_DELAY:
                        point["max_delay"] = bucket_data.get("_max_delay")
                    elif metric == TrendMetric.MIN_DELAY:
                        point["min_delay"] = bucket_data.get("_min_delay")
                    elif metric == TrendMetric.MAX_CONCURRENCY:
                        point["max_concurrency"] = bucket_data.get("_max_concurrency", 0)
                    elif metric == TrendMetric.SUCCESS_RATE:
                        success = bucket_data.get("_success_count", 0)
                        failed = bucket_data.get("_failed_count", 0)
                        total = success + failed
                        point["success_rate"] = round((success / total) * 100, 2) if total > 0 else 0

                trend.append(point)

            data.append({
                "queue_name": base_queue,
                "trend": trend
            })

        return data

    @staticmethod
    def _init_metric_values(metrics: List[TrendMetric]) -> Dict[str, Any]:
        values = {}
        for metric in metrics:
            if metric in [TrendMetric.TOTAL_COUNT, TrendMetric.AVG_DURATION, TrendMetric.AVG_DELAY]:
                values["_total_count"] = 0
            if metric in [TrendMetric.SUCCESS_COUNT, TrendMetric.SUCCESS_RATE]:
                values["_success_count"] = 0
            if metric in [TrendMetric.FAILED_COUNT, TrendMetric.SUCCESS_RATE]:
                values["_failed_count"] = 0
            if metric == TrendMetric.RETRY_COUNT:
                values["_retry_count"] = 0
            if metric in [TrendMetric.AVG_DURATION, TrendMetric.MAX_DURATION, TrendMetric.MIN_DURATION]:
                values["_total_duration"] = 0.0
                values["_max_duration"] = None
                values["_min_duration"] = None
            if metric in [TrendMetric.AVG_DELAY, TrendMetric.MAX_DELAY, TrendMetric.MIN_DELAY]:
                values["_total_delay"] = 0.0
                values["_max_delay"] = None
                values["_min_delay"] = None
            if metric == TrendMetric.MAX_CONCURRENCY:
                values["_max_concurrency"] = 0
        return values

    @staticmethod
    def _aggregate_metric_row(
        bucket_data: Dict[str, Any],
        row: Any,
        metric_fields: Dict[str, Any],
        metrics: List[TrendMetric]
    ):
        if "total_count" in metric_fields:
            bucket_data["_total_count"] = bucket_data.get("_total_count", 0) + (row.total_count or 0)
        if "success_count" in metric_fields:
            bucket_data["_success_count"] = bucket_data.get("_success_count", 0) + (row.success_count or 0)
        if "failed_count" in metric_fields:
            bucket_data["_failed_count"] = bucket_data.get("_failed_count", 0) + (row.failed_count or 0)
        if "retry_count" in metric_fields:
            bucket_data["_retry_count"] = bucket_data.get("_retry_count", 0) + (row.retry_count or 0)

        if "total_duration" in metric_fields:
            bucket_data["_total_duration"] = bucket_data.get("_total_duration", 0) + (row.total_duration or 0)
        if "total_delay" in metric_fields:
            bucket_data["_total_delay"] = bucket_data.get("_total_delay", 0) + (row.total_delay or 0)

        if "max_duration" in metric_fields and row.max_duration is not None:
            current = bucket_data.get("_max_duration")
            bucket_data["_max_duration"] = max(current, row.max_duration) if current is not None else row.max_duration
        if "max_delay" in metric_fields and row.max_delay is not None:
            current = bucket_data.get("_max_delay")
            bucket_data["_max_delay"] = max(current, row.max_delay) if current is not None else row.max_delay
        if "running_concurrency" in metric_fields:
            bucket_data["_max_concurrency"] = max(bucket_data.get("_max_concurrency", 0), row.running_concurrency or 0)

        if "min_duration" in metric_fields and row.min_duration is not None:
            current = bucket_data.get("_min_duration")
            bucket_data["_min_duration"] = min(current, row.min_duration) if current is not None else row.min_duration
        if "min_delay" in metric_fields and row.min_delay is not None:
            current = bucket_data.get("_min_delay")
            bucket_data["_min_delay"] = min(current, row.min_delay) if current is not None else row.min_delay

    @classmethod
    async def _query_from_aggregation_table(
        cls,
        pg_session: AsyncSession,
        namespace: str,
        queues: List[str],
        start_time: datetime,
        end_time: datetime,
        interval_seconds: int
    ) -> List[Dict[str, Any]]:
        stmt = select(
            TaskMetricsMinute.queue,
            TaskMetricsMinute.time_bucket,
            TaskMetricsMinute.task_count
        ).where(
            and_(
                TaskMetricsMinute.namespace == namespace,
                TaskMetricsMinute.queue.in_(queues),
                TaskMetricsMinute.time_bucket >= start_time,
                TaskMetricsMinute.time_bucket <= end_time
            )
        ).order_by(
            TaskMetricsMinute.queue,
            TaskMetricsMinute.time_bucket
        )

        result = await pg_session.execute(stmt)
        rows = result.fetchall()

        time_buckets = []
        if interval_seconds > 60:
            start_timestamp = int(start_time.timestamp() / interval_seconds) * interval_seconds
            current = datetime.fromtimestamp(start_timestamp, tz=timezone.utc)
            end_timestamp = int(end_time.timestamp() / interval_seconds) * interval_seconds
            end_aligned = datetime.fromtimestamp(end_timestamp, tz=timezone.utc)
        else:
            current = start_time.replace(second=0, microsecond=0)
            end_aligned = end_time.replace(second=0, microsecond=0)

        while current <= end_aligned:
            time_buckets.append(current)
            current += timedelta(seconds=interval_seconds)

        queue_data_map = {}
        for queue_name in queues:
            queue_data_map[queue_name] = {bucket: 0 for bucket in time_buckets}

        for row in rows:
            queue_name = row.queue
            time_bucket = row.time_bucket
            count = row.task_count

            if interval_seconds > 60:
                bucket_timestamp = int(time_bucket.timestamp() / interval_seconds) * interval_seconds
                aligned_bucket = datetime.fromtimestamp(bucket_timestamp, tz=timezone.utc)

                if queue_name in queue_data_map and aligned_bucket in queue_data_map[queue_name]:
                    queue_data_map[queue_name][aligned_bucket] += count
            else:
                if queue_name in queue_data_map and time_bucket in queue_data_map[queue_name]:
                    queue_data_map[queue_name][time_bucket] = count

        data = []
        for queue_name in queues:
            trend = []
            for bucket in sorted(time_buckets):
                trend.append({
                    "time": bucket.isoformat(),
                    "count": queue_data_map[queue_name].get(bucket, 0)
                })

            data.append({
                "queue_name": queue_name,
                "trend": trend
            })

        return data

    @classmethod
    async def _query_from_tasks_table(
        cls,
        pg_session: AsyncSession,
        namespace: str,
        queues: List[str],
        start_time: datetime,
        end_time: datetime,
        interval_seconds: int
    ) -> List[Dict[str, Any]]:
        stmt = select(
            Task.queue,
            Task.created_at
        ).where(
            and_(
                Task.namespace == namespace,
                Task.queue.in_(queues),
                Task.created_at >= start_time,
                Task.created_at <= end_time
            )
        ).order_by(Task.created_at)

        result = await pg_session.execute(stmt)
        tasks = result.fetchall()

        time_buckets = []
        current = start_time
        while current <= end_time:
            time_buckets.append(current)
            current += timedelta(seconds=interval_seconds)

        queue_data_map = {}
        for queue_name in queues:
            queue_data_map[queue_name] = {bucket: 0 for bucket in time_buckets}

        for row in tasks:
            queue_name = row.queue
            created_at = row.created_at

            bucket_timestamp = int(created_at.timestamp() / interval_seconds) * interval_seconds
            bucket = datetime.fromtimestamp(bucket_timestamp, tz=timezone.utc)

            if queue_name in queue_data_map and bucket in queue_data_map[queue_name]:
                queue_data_map[queue_name][bucket] += 1

        data = []
        for queue_name in queues:
            trend = []
            for bucket in sorted(time_buckets):
                trend.append({
                    "time": bucket.isoformat(),
                    "count": queue_data_map[queue_name].get(bucket, 0)
                })

            data.append({
                "queue_name": queue_name,
                "trend": trend
            })

        return data

    @classmethod
    async def get_task_runs_list(
        cls,
        namespace: str,
        pg_session: AsyncSession,
        registry,
        queue: str,
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        time_range: Optional[str] = None,
        where_clause: Optional[str] = None,
        page: int = 1,
        page_size: int = 20,
        sort_field: str = "created_at",
        sort_order: str = "desc"
    ) -> Dict[str, Any]:
        from sqlalchemy import desc, asc, text

        try:
            time_range_result = cls._resolve_time_range(
                start_time=start_time,
                end_time=end_time,
                time_range=time_range,
                default_range="1h"
            )
            start_time = time_range_result.start_time
            end_time = time_range_result.end_time

            queue_result = await cls._resolve_queues([queue], registry, expand_priority=True)
            all_priority_queues = queue_result.all_priority_queues

            logger.info(
                f"任务执行记录查询: namespace={namespace}, queue={queue}, "
                f"priority_queues={all_priority_queues}, "
                f"time_range={start_time} to {end_time}, "
                f"where_clause={where_clause}, page={page}, page_size={page_size}"
            )

            if not all_priority_queues:
                return {
                    "success": True,
                    "data": [],
                    "total": 0,
                    "page": page,
                    "page_size": page_size,
                    "time_range": {
                        "start_time": start_time.isoformat(),
                        "end_time": end_time.isoformat()
                    }
                }

            if where_clause:
                forbidden_keywords = [
                    'DROP', 'DELETE', 'UPDATE', 'INSERT', 'TRUNCATE', 'ALTER', 'CREATE',
                    'GRANT', 'REVOKE', 'EXEC', 'EXECUTE', 'UNION', 'INTO',
                    'GROUP BY', 'ORDER BY', 'HAVING', 'LIMIT', 'OFFSET',
                    '--', '/*', '*/', ';'
                ]
                upper_clause = where_clause.upper()
                for keyword in forbidden_keywords:
                    if keyword in upper_clause:
                        raise ValueError(f"WHERE 子句包含不允许的关键字: {keyword}")

            conditions = [
                Task.namespace == namespace,
                Task.queue.in_(all_priority_queues),
                Task.created_at >= start_time,
                Task.created_at <= end_time
            ]

            if where_clause and where_clause.strip():
                conditions.append(text(where_clause.strip()))

            count_stmt = select(func.count(Task.stream_id)).select_from(
                Task
            ).outerjoin(
                TaskRun, Task.stream_id == TaskRun.stream_id
            ).where(and_(*conditions))

            count_result = await pg_session.execute(count_stmt)
            total = count_result.scalar() or 0

            query_stmt = select(
                Task.stream_id,
                Task.queue,
                Task.namespace,
                Task.scheduled_task_id,
                Task.payload,
                Task.priority,
                Task.delay,
                Task.created_at,
                Task.trigger_time,
                Task.source,
                Task.task_metadata,
                TaskRun.task_name,
                TaskRun.status,
                TaskRun.result,
                TaskRun.error,
                TaskRun.started_at,
                TaskRun.completed_at,
                TaskRun.retries,
                TaskRun.duration,
                TaskRun.consumer
            ).select_from(
                Task
            ).outerjoin(
                TaskRun, Task.stream_id == TaskRun.stream_id
            ).where(and_(*conditions))

            sort_column = None
            if sort_field in ['stream_id', 'queue', 'namespace', 'priority', 'delay',
                              'created_at', 'trigger_time', 'source']:
                sort_column = getattr(Task, sort_field, None)
            elif sort_field in ['task_name', 'status', 'started_at', 'completed_at',
                                'retries', 'duration', 'consumer']:
                sort_column = getattr(TaskRun, sort_field, None)

            if sort_column is not None:
                if sort_order.lower() == 'desc':
                    query_stmt = query_stmt.order_by(desc(sort_column))
                else:
                    query_stmt = query_stmt.order_by(asc(sort_column))
            else:
                query_stmt = query_stmt.order_by(desc(Task.created_at))

            offset = (page - 1) * page_size
            query_stmt = query_stmt.offset(offset).limit(page_size)

            result = await pg_session.execute(query_stmt)
            rows = result.fetchall()

            data = []
            for row in rows:
                record = {
                    "stream_id": row.stream_id,
                    "namespace": row.namespace,
                    "scheduled_task_id": row.scheduled_task_id,
                    "payload": row.payload,
                    "priority": row.priority,
                    "delay": row.delay,
                    "created_at": row.created_at.isoformat() if row.created_at else None,
                    "trigger_time": row.trigger_time,
                    "source": row.source,
                    "metadata": row.task_metadata,
                    "task_name": row.task_name,
                    "status": row.status,
                    "result": row.result,
                    "error": row.error,
                    "started_at": row.started_at,
                    "completed_at": row.completed_at,
                    "retries": row.retries,
                    "duration": row.duration,
                    "consumer": row.consumer
                }
                data.append(record)

            return {
                "success": True,
                "data": data,
                "total": total,
                "page": page,
                "page_size": page_size,
                "queue": queue,  
                "time_range": {
                    "start_time": start_time.isoformat(),
                    "end_time": end_time.isoformat()
                }
            }

        except Exception as e:
            logger.error(f"获取任务执行记录失败: {e}", exc_info=True)
            return {
                "success": False,
                "error": str(e),
                "data": [],
                "total": 0,
                "page": page,
                "page_size": page_size
            }



"""
Webhook 相关的 Pydantic 模型

用于接收第三方平台的 webhook 回调通知
"""
from pydantic import BaseModel, Field
from typing import Optional, Dict, Any, List
from datetime import datetime
from enum import Enum


class WebhookStatus(str, Enum):
    """Webhook 处理状态"""
    PENDING = "pending"
    SUCCESS = "success"
    FAILED = "failed"


class WebhookReceiveResponse(BaseModel):
    """
    接收 webhook 通知的响应模型
    """
    success: bool = Field(..., description="是否成功接收")
    message: str = Field(..., description="响应消息")
    callback_id: str = Field(..., description="回调 ID")

    class Config:
        json_schema_extra = {
            "example": {
                "success": True,
                "message": "Webhook received successfully",
                "callback_id": "cb_abc123"
            }
        }


class WebhookInfo(BaseModel):
    """
    Webhook 记录信息
    """
    id: int = Field(..., description="Webhook 记录 ID")
    namespace: str = Field(..., description="命名空间")
    callback_id: str = Field(..., description="回调 ID")
    payload: Dict[str, Any] = Field(..., description="原始 webhook 载荷")
    status: WebhookStatus = Field(..., description="处理状态")
    received_at: datetime = Field(..., description="接收时间")


class WebhookListResponse(BaseModel):
    """
    Webhook 列表响应
    """
    success: bool = Field(True, description="是否成功")
    data: List[WebhookInfo] = Field(default_factory=list, description="Webhook 记录列表")
    total: int = Field(..., description="总记录数")
    page: int = Field(..., description="当前页码")
    page_size: int = Field(..., description="每页大小")
